@@ -1,3 +1,23 @@
+/* -*- mode: C; mode: fold -*-
+Copyright (C) 2010 John E. Davis
+
+This file is part of the S-Lang Library.
+
+The S-Lang Library is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License as
+published by the Free Software Foundation; either version 2 of the
+License, or (at your option) any later version.
+
+The S-Lang Library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this library; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+USA.
+*/
 #include "config.h"
 
 #include <stdio.h>
@@ -61,7 +81,7 @@ static SLang_MMT_Type *allocate_pcre_type (pcre *p, pcre_extra *extra)
      {
 	free_pcre_type (pt);
 	return NULL;
-     }   
+     }
    return mmt;
 }
 
@@ -78,7 +98,7 @@ static int _pcre_compile_1 (char *pattern, int options)
    p = pcre_compile (pattern, options, &err, &erroffset, table);
    if (NULL == p)
      {
-	SLang_verror (SL_Parse_Error, "Error compiling pattern '%s' at offset %d: %s", 
+	SLang_verror (SL_Parse_Error, "Error compiling pattern '%s' at offset %d: %s",
 		      pattern, erroffset, err);
 	return -1;
      }
@@ -91,7 +111,7 @@ static int _pcre_compile_1 (char *pattern, int options)
 	pcre_free (p);
 	return -1;
      }
-   
+
    if (NULL == (mmt = allocate_pcre_type (p, extra)))
      {
 	pcre_free ((char *) p);
@@ -111,7 +131,7 @@ static void _pcre_compile (void)
 {
    char *pattern;
    int options = 0;
-   
+
    switch (SLang_Num_Function_Args)
      {
       case 2:
@@ -127,16 +147,12 @@ static void _pcre_compile (void)
    SLang_free_slstring (pattern);
 }
 
-
-
 /* returns number of matches */
-static int _pcre_exec_1 (PCRE_Type *pt, char *str, int pos, int options)
+static int _pcre_exec_1 (PCRE_Type *pt, char *str, unsigned int len, int pos, int options)
 {
    int rc;
-   unsigned int len;
-   
+
    pt->num_matches = 0;
-   len = strlen (str);
    if ((unsigned int) pos > len)
      return 0;
 
@@ -160,6 +176,8 @@ static int _pcre_exec (void)
    PCRE_Type *p;
    SLang_MMT_Type *mmt;
    char *str;
+   SLang_BString_Type *bstr = NULL;
+   unsigned int len;
    int pos = 0;
    int options = 0;
    int ret = -1;
@@ -169,39 +187,64 @@ static int _pcre_exec (void)
       case 4:
 	if (-1 == SLang_pop_integer (&options))
 	  return -1;
+	/* drop */
       case 3:
+	/* drop */
 	if (-1 == SLang_pop_integer (&pos))
 	  return -1;
+	/* drop */
       default:
-	if (-1 == SLang_pop_slstring (&str))
-	  return -1;
+	switch (SLang_peek_at_stack())
+	  {
+	   case SLANG_STRING_TYPE:
+	     if (-1 == SLang_pop_slstring (&str))
+	       return -1;
+	     len = strlen (str);
+	     break;
 
-	if (NULL == (mmt = SLang_pop_mmt (PCRE_Type_Id)))
-	  goto free_and_return;
-	p = (PCRE_Type *)SLang_object_from_mmt (mmt);
+	   case SLANG_BSTRING_TYPE:
+	   default:
+	     if (-1 == SLang_pop_bstring(&bstr))
+	       return -1;
+	     str = (char *)SLbstring_get_pointer(bstr, &len);
+	     if (str == NULL)
+	       {
+		  SLbstring_free (bstr);
+		  return -1;
+	       }
+	     break;
+	  }
      }
-   ret = _pcre_exec_1 (p, str, pos, options);
-   
-   free_and_return:
-   SLang_free_slstring (str);
+
+   if (NULL == (mmt = SLang_pop_mmt (PCRE_Type_Id)))
+     goto free_and_return;
+   p = (PCRE_Type *)SLang_object_from_mmt (mmt);
+
+   ret = _pcre_exec_1 (p, str, len, pos, options);
+
+free_and_return:
+
    SLang_free_mmt (mmt);
+   if (bstr != NULL)
+     SLbstring_free (bstr);
+   else
+     SLang_free_slstring (str);
    return ret;
 }
 
-
-static int get_nth_start_stop (PCRE_Type *pt, unsigned int n, 
+static int get_nth_start_stop (PCRE_Type *pt, unsigned int n,
 			       unsigned int *a, unsigned int *b)
 {
    int start, stop;
 
-   if (n >= pt->num_matches) 
+   if (n >= pt->num_matches)
      return -1;
-   
+
    start = pt->ovector[2*n];
    stop = pt->ovector[2*n+1];
    if ((start < 0) || (stop < start))
      return -1;
-   
+
    *a = (unsigned int) start;
    *b = (unsigned int) stop;
    return 0;
@@ -219,10 +262,10 @@ static void _pcre_nth_match (PCRE_Type *pt, int *np)
 	SLang_push_null ();
 	return;
      }
-   
+
    if (NULL == (at = SLang_create_array (SLANG_INT_TYPE, 0, NULL, &two, 1)))
      return;
-   
+
    data = (int *)at->data;
    data[0] = (int)start;
    data[1] = (int)stop;
@@ -233,7 +276,7 @@ static void _pcre_nth_substr (PCRE_Type *pt, char *str, int *np)
 {
    unsigned int start, stop;
    unsigned int len;
-   
+
    len = strlen (str);
 
    if ((-1 == get_nth_start_stop (pt, (unsigned int) *np, &start, &stop))
@@ -242,7 +285,7 @@ static void _pcre_nth_substr (PCRE_Type *pt, char *str, int *np)
 	SLang_push_null ();
 	return;
      }
-   
+
    str = SLang_create_nslstring (str + start, stop - start);
    (void) SLang_push_string (str);
    SLang_free_slstring (str);
@@ -277,7 +320,7 @@ static char *_slang_to_pcre (char *slpattern)
    pattern = SLmalloc (3*len + 1);
    if (pattern == NULL)
      return NULL;
-   
+
    p = pattern;
    s = slpattern;
    in_bracket = 0;
@@ -299,7 +342,7 @@ static char *_slang_to_pcre (char *slpattern)
 	     in_bracket = 1;
 	     *p++ = ch;
 	     break;
-	     
+
 	   case ']':
 	     in_bracket = 0;
 	     *p++ = ch;
@@ -312,7 +355,7 @@ static char *_slang_to_pcre (char *slpattern)
 		case 0:
 		  s--;
 		  break;
-		  
+
 		case '<':
 		case '>':
 		  *p++ = '\\'; *p++ = 'b';
@@ -331,20 +374,20 @@ static char *_slang_to_pcre (char *slpattern)
 		case 'c':
 		  *p++ = '('; *p++ = '?'; *p++ = '-'; *p++ = 'i'; *p++ = ')';
 		  break;
-		  
+
 		default:
 		  *p++ = '\\';
 		  *p++ = ch;
 	       }
 	     break;
-	     
+
 	   default:
 	     *p++ = ch;
 	     break;
 	  }
      }
    *p = 0;
-   
+
    s = SLang_create_slstring (pattern);
    SLfree (pattern);
    return s;
@@ -362,7 +405,7 @@ static void destroy_pcre (SLtype type, VOID_STAR f)
 {
    PCRE_Type *pt;
    (void) type;
-   
+
    pt = (PCRE_Type *) f;
    if (pt->extra != NULL)
      pcre_free ((char *) pt->extra);
@@ -371,7 +414,7 @@ static void destroy_pcre (SLtype type, VOID_STAR f)
    free_pcre_type (pt);
 }
 
-#define DUMMY_PCRE_TYPE ((unsigned int)-1)
+#define DUMMY_PCRE_TYPE ((SLtype)-1)
 #define P DUMMY_PCRE_TYPE
 #define I SLANG_INT_TYPE
 #define V SLANG_VOID_TYPE
@@ -398,7 +441,7 @@ static SLang_IConstant_Type PCRE_Consts [] =
    MAKE_ICONSTANT("PCRE_MULTILINE", PCRE_MULTILINE),
    MAKE_ICONSTANT("PCRE_UNGREEDY", PCRE_UNGREEDY),
    MAKE_ICONSTANT("PCRE_UTF8", PCRE_UTF8),
-   
+
    /* exec options */
    MAKE_ICONSTANT("PCRE_NOTBOL", PCRE_NOTBOL),
    MAKE_ICONSTANT("PCRE_NOTEOL", PCRE_NOTEOL),
@@ -411,31 +454,6 @@ static SLang_IConstant_Type PCRE_Consts [] =
 #undef V
 #undef S
 
-static void patchup_intrinsic_table (SLang_Intrin_Fun_Type *table, 
-				     SLtype dummy, SLtype type)
-{
-   while (table->name != NULL)
-     {
-	unsigned int i, nargs;
-	SLtype *args;
-	
-	nargs = table->num_args;
-	args = table->arg_types;
-	for (i = 0; i < nargs; i++)
-	  {
-	     if (args[i] == dummy)
-	       args[i] = type;
-	  }
-	
-	/* For completeness */
-	if (table->return_type == dummy)
-	  table->return_type = type;
-
-	table++;
-     }
-}
-
-
 static int register_pcre_type (void)
 {
    SLang_Class_Type *cl;
@@ -456,7 +474,8 @@ static int register_pcre_type (void)
      return -1;
 
    PCRE_Type_Id = SLclass_get_class_id (cl);
-   patchup_intrinsic_table (PCRE_Intrinsics, DUMMY_PCRE_TYPE, PCRE_Type_Id);
+   if (-1 == SLclass_patch_intrin_fun_table1 (PCRE_Intrinsics, DUMMY_PCRE_TYPE, PCRE_Type_Id))
+     return -1;
 
    return 0;
 }
@@ -490,7 +509,6 @@ int init_pcre_module_ns (char *ns_name)
    return 0;
 }
 
-
 /* This function is optional */
 void deinit_pcre_module (void)
 {
