@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2005-2009 John E. Davis
+Copyright (C) 2005-2010 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -58,7 +58,6 @@ USA.
 # include <readline/history.h>
 #endif
 
-
 static int Use_Readline;
 static int Slsh_Quit = 0;
 static SLang_Load_Type *Readline_Load_Object;
@@ -105,12 +104,50 @@ static void sig_sigtstp (int sig)
    reset_tty ();
    kill(getpid(),SIGSTOP);
    init_tty ();
-   if (Active_Rline_Info != NULL) 
-     SLrline_redraw (Active_Rline_Info);
+   if (Active_Rline_Info != NULL)
+     {
+	SLrline_set_display_width (Active_Rline_Info, SLtt_Screen_Cols);
+	SLrline_redraw (Active_Rline_Info);
+     }
    SLsig_unblock_signals ();
 }
 # endif
 
+#ifdef SIGWINCH
+static int Want_Window_Size_Change = 0;
+static void sig_winch_handler (int sig)
+{
+   sig = errno;
+   Want_Window_Size_Change = 1;
+   SLsignal_intr (SIGWINCH, sig_winch_handler);
+   errno = sig;
+}
+
+static int screen_size_changed_hook (VOID_STAR cd_unused)
+{
+   (void) cd_unused;
+   if (Want_Window_Size_Change)
+     {
+	Want_Window_Size_Change = 0;
+	if (Active_Rline_Info != NULL)
+	  {
+	     SLtt_get_screen_size ();
+	     SLrline_set_display_width (Active_Rline_Info, SLtt_Screen_Cols);
+	  }
+     }
+   return 0;
+}
+#endif
+
+static int add_sigwinch_handlers (void)
+{
+#ifdef SIGWINCH
+   (void) SLang_add_interrupt_hook (screen_size_changed_hook, NULL);
+   (void) SLsignal_intr (SIGWINCH, sig_winch_handler);
+#endif
+   return 0;
+}
+
 # ifdef REAL_UNIX_SYSTEM
 /* This hook if a signal occurs while waiting for input. */
 static int getkey_intr_hook (void)
@@ -147,6 +184,9 @@ static void init_tty (void)
 # ifdef REAL_UNIX_SYSTEM
    SLang_getkey_intr_hook = getkey_intr_hook;
 # endif
+   (void) add_sigwinch_handlers ();
+
+   SLtt_get_screen_size ();
 
 # if SYSTEM_SUPPORTS_SIGNALS
    SLtty_set_suspend_state (1);
@@ -193,8 +233,9 @@ static int open_readline (char *name)
 #if USE_GNU_READLINE
    return 0;
 #else
+   unsigned int flags = SL_RLINE_BLINK_MATCH|SL_RLINE_USE_MULTILINE;
    close_readline ();
-   if (NULL == (Rline_Info = SLrline_open2 (name, SLtt_Screen_Cols, SL_RLINE_BLINK_MATCH)))
+   if (NULL == (Rline_Info = SLrline_open2 (name, SLtt_Screen_Cols, flags)))
      return -1;
    return 0;
 #endif
@@ -205,6 +246,7 @@ static void redisplay_dummy (void)
 {
 }
 #endif
+
 static char *read_input_line (SLang_RLine_Info_Type *rline, char *prompt, int noecho)
 {
    char *line;
@@ -228,7 +270,7 @@ static char *read_input_line (SLang_RLine_Info_Type *rline, char *prompt, int no
 	       }
 #endif
 	  }
-	
+
 	line = buf;
 	while (NULL == fgets (buf, sizeof (buf), stdin))
 	  {
@@ -279,6 +321,7 @@ static char *read_input_line (SLang_RLine_Info_Type *rline, char *prompt, int no
 #else
    SLtt_get_screen_size ();
    SLrline_set_display_width (rline, SLtt_Screen_Cols);
+   (void) add_sigwinch_handlers ();
    Active_Rline_Info = rline;
    (void) SLrline_set_echo (rline, (noecho == 0));
    line = SLrline_read_line (rline, prompt, NULL);
@@ -293,14 +336,13 @@ static char *read_input_line (SLang_RLine_Info_Type *rline, char *prompt, int no
    return line;
 }
 
-
 static int save_input_line (SLang_RLine_Info_Type *rline, char *line)
 {
    char *p;
 
    if (line == NULL)
      return 0;
-   
+
    p = line;
    while ((*p == ' ') || (*p == '\t') || (*p == '\n'))
      p++;
@@ -329,9 +371,9 @@ static void set_prompt_hook (void)
    else if (NULL == (h = SLang_pop_function ()))
      return;
 
-   if (Prompt_Hook != NULL) 
+   if (Prompt_Hook != NULL)
      SLang_free_function (Prompt_Hook);
-   
+
    Prompt_Hook = h;
 }
 
@@ -372,7 +414,7 @@ static char *get_input_line (SLang_Load_Type *x)
    else
      prompt = "       ";
 
-   if (parse_level == 0) 
+   if (parse_level == 0)
      {
 	if (-1 == SLang_run_hooks ("slsh_interactive_before_hook", 0))
 	  {
@@ -386,7 +428,7 @@ static char *get_input_line (SLang_Load_Type *x)
    if (free_prompt)
      SLang_free_slstring (prompt);
 
-   if ((line == NULL) 
+   if ((line == NULL)
        && (parse_level == 0)
        && (SLang_get_error() == 0))
      {
@@ -403,7 +445,7 @@ static char *get_input_line (SLang_Load_Type *x)
    (void) SLang_run_hooks ("slsh_interactive_after_hook", 1, line);
 
    (void) save_input_line (Rline_Info, line);
-   
+
    return line;
 }
 
@@ -449,7 +491,7 @@ static char *read_using_readline (SLang_Load_Type *x)
 static void enable_keyboard_interrupt (void)
 {
    static int is_enabled = 0;
-   
+
    if (is_enabled == 0)
      {
 	(void) SLang_set_abort_signal (NULL);
@@ -463,7 +505,7 @@ static void close_interactive (void)
 
    if (Readline_Load_Object == NULL)
      return;
-     
+
    SLdeallocate_load_type (Readline_Load_Object);
    Readline_Load_Object = NULL;
 #if !SYSTEM_SUPPORTS_SIGNALS
@@ -499,11 +541,10 @@ static int open_interactive (void)
    return 0;
 }
 
-
 static int init_readline (char *appname)
 {
    static int inited = 0;
-   
+
    if (inited)
      return 0;
 
@@ -563,7 +604,6 @@ int slsh_interactive (void)
    return 0;
 }
 
-
 static SLang_RLine_Info_Type *Intrinsic_Rline_Info;
 #if USE_SLANG_READLINE
 static void close_intrinsic_readline (void)
@@ -575,7 +615,7 @@ static void close_intrinsic_readline (void)
      }
 }
 #endif
-	
+
 static int readline_intrinsic_internal (SLang_RLine_Info_Type *rli, char *prompt, int noecho)
 {
    char *line;
@@ -656,7 +696,7 @@ static void new_slrline_intrinsic (char *name)
 
    if (NULL == (rli = SLrline_open2 (name, SLtt_Screen_Cols, SL_RLINE_BLINK_MATCH)))
      return;
-   
+
    if (NULL == (mmt = SLang_create_mmt (Rline_Type_Id, (VOID_STAR) rli)))
      {
 	SLrline_close (rli);
@@ -674,6 +714,8 @@ static void init_readline_intrinsic (char *appname)
 
 static SLang_Intrin_Fun_Type Intrinsics [] =
 {
+   MAKE_INTRINSIC_0("__rline_init_tty", init_tty, VOID_TYPE),
+   MAKE_INTRINSIC_0("__rline_reset_tty", reset_tty, VOID_TYPE),
    MAKE_INTRINSIC_S("slsh_readline_init", init_readline_intrinsic, VOID_TYPE),
    MAKE_INTRINSIC_S("slsh_readline_new", new_slrline_intrinsic, VOID_TYPE),
    MAKE_INTRINSIC_S("slsh_readline", readline_intrinsic, VOID_TYPE),
@@ -687,7 +729,7 @@ static void destroy_rline (SLtype type, VOID_STAR f)
 {
    SLang_RLine_Info_Type *rli;
    (void) type;
-   
+
    rli = (SLang_RLine_Info_Type *) f;
    if (rli != NULL)
      SLrline_close (rli);
@@ -717,7 +759,6 @@ static int register_rline_type (void)
    return 0;
 }
 
-
 int slsh_init_readline_intrinsics ()
 {
    if (-1 == register_rline_type ())
@@ -725,7 +766,7 @@ int slsh_init_readline_intrinsics ()
 
    if (-1 == SLadd_intrin_fun_table (Intrinsics, NULL))
      return -1;
-   
+
    return 0;
 }
 
