@@ -3,7 +3,7 @@
 /* header file for S-Lang internal structures that users do not (should not)
    need.  Use slang.h for that purpose. */
 /*
-Copyright (C) 2004-2009 John E. Davis
+Copyright (C) 2004-2010 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -20,7 +20,7 @@ General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-USA.  
+USA.
 */
 
 /* #include "config.h" */
@@ -92,8 +92,8 @@ typedef enum
    SLANG_BC_LOBJPTR		= 0x2A,
    SLANG_BC_GOBJPTR		= 0x2B,
    SLANG_BC_FIELD_REF		= 0x2C,
-   SLANG_BC_DEREF_FUN_CALL	= 0x2D,
-   SLANG_BC_UNUSED_0x2E		= 0x2E,
+   SLANG_BC_OBSOLETE_DEREF_FUN_CALL	= 0x2D,
+   SLANG_BC_DEREF_FUN_CALL	= 0x2E,
    SLANG_BC_UNUSED_0x2F		= 0x2F,
    SLANG_BC_UNUSED_0x30		= 0x30,
    SLANG_BC_UNUSED_0x31		= 0x31,
@@ -111,7 +111,7 @@ typedef enum
    SLANG_BC_UNUSED_0x3D		= 0x3D,
    SLANG_BC_UNUSED_0x3E		= 0x3E,
    SLANG_BC_UNUSED_0x3F		= 0x3F,
-   
+
    /* byte codes for literals */
    SLANG_BC_LITERAL		= 0x40,           /* constant objects */
    SLANG_BC_LITERAL_INT		= 0x41,
@@ -131,7 +131,7 @@ typedef enum
    SLANG_BC_UNUSED_0x4D		= 0x4D,
    SLANG_BC_UNUSED_0x4E		= 0x4E,
    SLANG_BC_UNUSED_0x4F		= 0x4F,
-   
+
    /* Unary/Binary operation codes */
    SLANG_BC_UNARY		= 0x50,
    SLANG_BC_BINARY		= 0x51,
@@ -149,7 +149,7 @@ typedef enum
    SLANG_BC_UNUSED_0x5D		= 0x5D,
    SLANG_BC_UNUSED_0x5E		= 0x5E,
    SLANG_BC_UNUSED_0x5F		= 0x5F,
-   
+
    /* byte codes associated with keywords and blocks */
    SLANG_BC_TMP			= 0x60,
    SLANG_BC_EXCH		= 0x61,
@@ -167,12 +167,11 @@ typedef enum
    SLANG_BC_X_USER2		= 0x6D,
    SLANG_BC_X_USER3		= 0x6E,
    SLANG_BC_X_USER4		= 0x6F,
-   
+
    /* byte codes for dealing with the frame pointer and arg list */
    SLANG_BC_CALL_DIRECT		= 0x70,
    SLANG_BC_CALL_DIRECT_FRAME	= 0x71,
-   /* SLANG_BC_ARG_LVARIABLE	= 0x72, */
-   SLANG_BC_UNUSED_0x72		= 0x72,
+   SLANG_BC_CALL_DIRECT_NARGS	= 0x72,
    SLANG_BC_EARG_LVARIABLE	= 0x73,
 #define USE_BC_LINE_NUM	0	       /* used in slang.c */
    /* SLANG_BC_LINE_NUM		= 0x74, */
@@ -188,7 +187,7 @@ typedef enum
    SLANG_BC_UNUSED_0x7D		= 0x7D,
    SLANG_BC_UNUSED_0x7E		= 0x7E,
    SLANG_BC_UNUSED_0x7F		= 0x7F,
-   
+
    /* These are used only when compiled with USE_COMBINED_BYTECODES */
    SLANG_BC_CALL_DIRECT_INTRINSIC= 0x80,
    SLANG_BC_INTRINSIC_CALL_DIRECT= 0x81,
@@ -256,7 +255,7 @@ typedef enum
    SLANG_BC_UNUSED_0xBE		= 0xBE,
    SLANG_BC_UNUSED_0xBF		= 0xBF,
 
-   /* The following do not actually occur in inner_interp.  They used 
+   /* The following do not actually occur in inner_interp.  They used
     * to signify the bytecode has been combined with another.
     */
    SLANG_BC_LVARIABLE_COMBINED	= 0xC0,
@@ -326,7 +325,7 @@ typedef enum
    SLANG_BC_UNUSED_0xFF		= 0xFF
 }
 _pSLang_BC_Type;
-   
+
 /* Byte-Code Sub Types (_BCST_) */
 
 /* These are sub_types of SLANG_BC_BLOCK */
@@ -344,8 +343,9 @@ _pSLang_BC_Type;
 #define SLANG_BCST_FOREVER	0x13
 #define SLANG_BCST_CFOR	0x14
 #define SLANG_BCST_DOWHILE	0x15
-#define SLANG_BCST_FOREACH	0x16
+#define SLANG_BCST_FOREACH	0x16   /* obsolete */
 #define SLANG_BCST_TRY		0x17
+#define SLANG_BCST_FOREACH_EARGS	0x18
 
 #define SLANG_BCST_IF		0x20
 #define SLANG_BCST_IFNOT	0x21
@@ -378,7 +378,6 @@ _pSLang_BC_Type;
 
 /* These use SLANG_PLUS, SLANG_MINUS, SLANG_PLUSPLUS, etc... */
 
-
 #define LONG_IS_INT (SIZEOF_INT == SIZEOF_LONG)
 #define LONG_IS_NOT_INT (SIZEOF_INT != SIZEOF_LONG)
 #define SHORT_IS_INT (SIZEOF_INT == SIZEOF_SHORT)
@@ -507,7 +506,6 @@ typedef unsigned long long _pSLuint64_Type;
 # endif
 #endif
 
-
 typedef union
 {
 #if SLANG_HAS_FLOAT
@@ -543,6 +541,16 @@ typedef struct _pSLang_Object_Type
 }
 SLang_Object_Type;
 
+typedef struct
+{
+   char *name;
+   SLang_Name_Type *next;
+   char name_type;
+
+   SLang_Object_Type obj;
+}
+SLang_Global_Var_Type;
+
 struct _pSLang_MMT_Type
 {
    SLtype data_type;	       /* int, string, etc... */
@@ -579,7 +587,12 @@ extern int _pSLstruct_push_field_ref (SLFUTURE_CONST char *);
 extern int _pSLstruct_push_field (SLang_Struct_Type *s, SLFUTURE_CONST char *name, int do_free);
 extern int _pSLstruct_pop_field (SLang_Struct_Type *s, SLFUTURE_CONST char *name, int do_free);
 
-int _pSLang_get_qualifiers (SLang_Struct_Type **);
+extern int _pSLang_get_qualifiers_intrin (SLang_Struct_Type **);
+
+/* The following qualifier functions will eventually be part of the public API */
+extern int _pSLang_qualifier_exists (SLCONST char *);
+extern int _pSLang_get_int_qualifier (SLCONST char *, int *, int);
+extern int _pSLang_get_string_qualifier (SLCONST char *, char **p, SLFUTURE_CONST char *);
 
 struct _pSLang_Ref_Type
 {
@@ -606,9 +619,10 @@ extern int _pSL_decrement_frame_pointer (void);
 
 extern int SLang_pop(SLang_Object_Type *);
 extern void SLang_free_object (SLang_Object_Type *);
-extern int _pSLanytype_typecast (SLtype, VOID_STAR, unsigned int,
-				SLtype, VOID_STAR);
+extern int _pSLanytype_typecast (SLtype, VOID_STAR, SLuindex_Type,
+				 SLtype, VOID_STAR);
 extern void _pSLstring_intrinsic (void);
+extern int _pSLformat_as_binary (unsigned int min_num_bits, int use_binary_prefix);
 
 #if 0
 /* NULL not allowed here */
@@ -626,12 +640,11 @@ extern char *_pSLcreate_via_alloced_slstring (char *, unsigned int);
 extern void _pSLunallocate_slstring (char *, unsigned int);
 extern int _pSLpush_alloced_slstring (char *, unsigned int);
 
-
 extern unsigned int _pSLstring_bytelen (SLstr_Type *);
 extern void _pSLang_free_slstring (SLstr_Type *);   /* slstring required and assumed */
 extern unsigned long _pSLstring_get_hash (SLstr_Type *s);   /* slstring required */
 
-typedef struct 
+typedef struct
 {
    char **buf;
    unsigned int max_num;
@@ -655,7 +668,6 @@ extern SLang_Array_Type *_pSLstrings_to_array (char **strs, unsigned int n);
 extern SLCONST char *_pSLstring_dup_slstring (SLCONST char *);
 extern int _pSLang_dup_and_push_slstring (SLCONST char *);
 
-
 extern int _pSLang_init_import (void);
 extern int _pSLinit_exceptions (void);
 
@@ -708,6 +720,29 @@ extern SLang_NameSpace_Type *_pSLns_find_object_namespace (SLang_Name_Type *nt);
 extern SLang_Name_Type *_pSLns_locate_name (SLang_NameSpace_Type *, SLCONST char *);
 extern SLang_NameSpace_Type *_pSLns_get_private_namespace (SLFUTURE_CONST char *name, SLFUTURE_CONST char *nsname);
 extern SLang_NameSpace_Type *_pSLns_create_namespace2 (SLFUTURE_CONST char *name, SLFUTURE_CONST char *nsname);
+extern void _pSLns_delete_namespaces (void);
+
+/* The long-long constants functions need to be fixed in slang.h.  This will
+ * have to wait for a major version change.
+ */
+#ifdef HAVE_LONG_LONG
+typedef struct
+{
+   SLFUTURE_CONST char *name;
+   SLang_Name_Type *next;
+   char name_type;
+
+   SLtype data_type;
+   long long value;
+}
+_pSLang_LLConstant_Type;
+SL_EXTERN int _pSLns_add_llconstant_table (SLang_NameSpace_Type *, _pSLang_LLConstant_Type *, SLFUTURE_CONST char *);
+SL_EXTERN int _pSLadd_llconstant_table (_pSLang_LLConstant_Type *, SLFUTURE_CONST char *);
+SL_EXTERN int _pSLns_add_llconstant (SLang_NameSpace_Type *, SLFUTURE_CONST char *, SLtype, long long);
+# define _pMAKE_LLCONSTANT_T(n,val,T) \
+    {(n),NULL, SLANG_LLCONSTANT, T, (long long)(val)}
+# define _pSLANG_END_LLCONST_TABLE _pMAKE_LLCONSTANT_T(NULL,0,0)
+#endif
 
 extern int _pSLang_Trace;
 extern char SLCONST *_pSLang_current_function_name (void);
@@ -757,8 +792,8 @@ extern int _pSLang_push_slstring (char *);
 
 extern SLtype _pSLarith_promote_type (SLtype);
 extern int _pSLarith_get_precedence (SLtype);
-extern int _pSLarith_typecast (SLtype, VOID_STAR, unsigned int,
-			      SLtype, VOID_STAR);
+extern int _pSLarith_typecast (SLtype, VOID_STAR, SLuindex_Type,
+			       SLtype, VOID_STAR);
 
 extern int SLang_push(SLang_Object_Type *);
 extern int SLadd_global_variable (SLCONST char *);
@@ -780,10 +815,19 @@ extern char *_pSLtt_tigetstr (SLterminfo_Type *, SLCONST char *);
 extern int _pSLtt_tigetnum (SLterminfo_Type *, SLCONST char *);
 extern int _pSLtt_tigetflag (SLterminfo_Type *, SLCONST char *);
 
+#if defined(REAL_UNIX_SYSTEM) || defined(VMS)
+extern void SLtt_init_keypad (void);
+extern void SLtt_deinit_keypad (void);
+#endif
+
 #if SLTT_HAS_NON_BCE_SUPPORT
 extern int _pSLtt_get_bce_color_offset (void);
 #endif
 extern void (*_pSLtt_color_changed_hook)(void);
+extern int _pSLtt_init_cmdline_mode (void);
+extern void _pSLtt_cmdline_mode_reset (void);
+
+extern int _pSLsmg_init_smg_cmdline (void);
 
 extern unsigned char SLang_Input_Buffer [SL_MAX_INPUT_BUFFER_LEN];
 
@@ -792,8 +836,8 @@ typedef struct SL_OOBinary_Type
    SLtype data_type;	       /* partner type for binary op */
 
     int (*binary_function)_PROTO((int,
-				 SLtype, VOID_STAR, unsigned int,
-				 SLtype, VOID_STAR, unsigned int,
+				 SLtype, VOID_STAR, SLuindex_Type,
+				 SLtype, VOID_STAR, SLuindex_Type,
 				 VOID_STAR));
 
    int (*binary_result) _PROTO((int, SLtype, SLtype, SLtype *));
@@ -806,7 +850,7 @@ typedef struct _pSL_Typecast_Type
    SLtype data_type;	       /* to_type */
    int allow_implicit;
 
-   int (*typecast)_PROTO((SLtype, VOID_STAR, unsigned int,
+   int (*typecast)_PROTO((SLtype, VOID_STAR, SLuindex_Type,
 			  SLtype, VOID_STAR));
    struct _pSL_Typecast_Type *next;
 }
@@ -856,18 +900,18 @@ struct _pSLang_Class_Type
 
    /* mul2, sign, etc... */
    int (*cl_unary_op_result_type)_PROTO((int, SLtype, SLtype *));
-   int (*cl_unary_op)_PROTO((int, SLtype, VOID_STAR, unsigned int, VOID_STAR));
+   int (*cl_unary_op)_PROTO((int, SLtype, VOID_STAR, SLuindex_Type, VOID_STAR));
 
 #if 0
    int (*cl_arith_unary_op_result_type)_PROTO((int, SLtype, SLtype *));
-   int (*cl_arith_unary_op)_PROTO((int, SLtype, VOID_STAR, unsigned int, VOID_STAR));
+   int (*cl_arith_unary_op)_PROTO((int, SLtype, VOID_STAR, SLuindex_Type, VOID_STAR));
 #endif
    int (*cl_app_unary_op_result_type)_PROTO((int, SLtype, SLtype *));
-   int (*cl_app_unary_op)_PROTO((int, SLtype, VOID_STAR, unsigned int, VOID_STAR));
+   int (*cl_app_unary_op)_PROTO((int, SLtype, VOID_STAR, SLuindex_Type, VOID_STAR));
 
    /* If this function is non-NULL, it will be called for sin, cos, etc... */
 
-   int (*cl_math_op)_PROTO((int, SLtype, VOID_STAR, unsigned int, VOID_STAR));
+   int (*cl_math_op)_PROTO((int, SLtype, VOID_STAR, SLuindex_Type, VOID_STAR));
    int (*cl_math_op_result_type)_PROTO((int, SLtype, SLtype *));
 
    SL_OOBinary_Type *cl_binary_ops;
@@ -880,7 +924,7 @@ struct _pSLang_Class_Type
    SLang_Struct_Type *cl_struct_def;
    int (*cl_dereference) _PROTO((SLtype, VOID_STAR));
 
-   /* cl_a* functions deal with functions that have array addresses.  For 
+   /* cl_a* functions deal with functions that have array addresses.  For
     * scalar and vector types, there is no difference.  However, for vector
     * types such as Complex, which is stored as double[2], the address is
     * the address of the first byte, and not the object itself.
@@ -891,9 +935,9 @@ struct _pSLang_Class_Type
    int (*cl_push_literal) _PROTO((SLtype, VOID_STAR));
    void (*cl_adestroy)_PROTO((SLtype, VOID_STAR));
    int (*cl_push_intrinsic)_PROTO((SLtype, VOID_STAR));
-   int (*cl_void_typecast)_PROTO((SLtype, VOID_STAR, unsigned int, SLtype, VOID_STAR));
+   int (*cl_void_typecast)_PROTO((SLtype, VOID_STAR, SLuindex_Type, SLtype, VOID_STAR));
 
-   int (*cl_anytype_typecast)_PROTO((SLtype, VOID_STAR, unsigned int, SLtype, VOID_STAR));
+   int (*cl_anytype_typecast)_PROTO((SLtype, VOID_STAR, SLuindex_Type, SLtype, VOID_STAR));
 
    /* Array access functions */
    int (*cl_aput) (SLtype, unsigned int);
@@ -901,7 +945,7 @@ struct _pSLang_Class_Type
    int (*cl_anew) (SLtype, unsigned int);
 
    /* length method */
-   int (*cl_length) (SLtype, VOID_STAR, unsigned int *);
+   int (*cl_length) (SLtype, VOID_STAR, SLuindex_Type *);
 
    /* foreach */
    SLang_Foreach_Context_Type *(*cl_foreach_open) (SLtype, unsigned int);
@@ -919,10 +963,10 @@ struct _pSLang_Class_Type
    int (*cl_fdwrite) (SLtype, int, VOID_STAR, unsigned int, unsigned int *);
 
    int (*cl_to_bool) (SLtype, int *);
-   
-   int (*cl_cmp)(SLtype, VOID_STAR, VOID_STAR, int *);   
+
+   int (*cl_cmp)(SLtype, VOID_STAR, VOID_STAR, int *);
    int (*cl_eqs)(SLtype, VOID_STAR, SLtype, VOID_STAR);
-   
+
    /* required for any object that takes advantage of __tmp optimization */
    void (*cl_inc_ref)(SLtype, VOID_STAR, int);
 
@@ -942,17 +986,17 @@ extern int _pSLclass_init (void);
 extern int _pSLclass_copy_class (SLtype, SLtype);
 
 extern int (*_pSLclass_get_typecast (SLtype, SLtype, int))
-(SLtype, VOID_STAR, unsigned int,
+(SLtype, VOID_STAR, SLuindex_Type,
  SLtype, VOID_STAR);
 
 extern int (*_pSLclass_get_binary_fun (int, SLang_Class_Type *, SLang_Class_Type *, SLang_Class_Type **, int))
 (int,
- SLtype, VOID_STAR, unsigned int,
- SLtype, VOID_STAR, unsigned int,
+ SLtype, VOID_STAR, SLuindex_Type,
+ SLtype, VOID_STAR, SLuindex_Type,
  VOID_STAR);
 
 extern int (*_pSLclass_get_unary_fun (int, SLang_Class_Type *, SLang_Class_Type **, int))
-(int, SLtype, VOID_STAR, unsigned int, VOID_STAR);
+(int, SLtype, VOID_STAR, SLuindex_Type, VOID_STAR);
 
 #if 0
 extern int _pSLclass_add_arith_unary_op (SLtype type,
@@ -981,7 +1025,7 @@ extern int _pSLarray1d_push_elem (SLang_Array_Type *at, SLindex_Type idx);
 #endif
 extern int _pSLarith_bin_op (SLang_Object_Type *, SLang_Object_Type *, int);
 
-/* Does not perform any range checking.  It is up to the caller. 
+/* Does not perform any range checking.  It is up to the caller.
  * If *atp!=NULL, then it is an array of indices.
  */
 extern int _pSLarray_pop_index (unsigned int num_elements, SLang_Array_Type **ind_atp, SLindex_Type *ind);
@@ -1045,8 +1089,8 @@ extern int _pSLarray_inline_array (void);
 extern int _pSLarray_wildcard_array (void);
 
 extern int
-_pSLarray_typecast (SLtype, VOID_STAR, unsigned int,
-		   SLtype, VOID_STAR, int);
+_pSLarray_typecast (SLtype, VOID_STAR, SLuindex_Type,
+		    SLtype, VOID_STAR, int);
 
 extern int _pSLarray_aput_transfer_elem (SLang_Array_Type *, SLindex_Type *,
 					VOID_STAR, size_t, int);
@@ -1084,13 +1128,13 @@ typedef struct _pSLtoken_String_List_Type
 _pSLtoken_String_List_Type;
 
 typedef unsigned char _pSLtok_Type;
-typedef struct 
+typedef struct
 {
    _pSLtok_Type type;			       /* [b]string_token */
    unsigned int num;
-   
+
    _pSLtoken_String_List_Type *list;
-   union 
+   union
      {
 	SLFUTURE_CONST char *s_val;		       /* (SLstring) concatenated strings */
 	SLang_BString_Type *b_val;     /* concatenated bstrings */
@@ -1106,7 +1150,7 @@ typedef struct _pSLang_Token_Type
      {
 	long long_val;
 	unsigned long ulong_val;
-#if HAVE_LONG_LONG
+#ifdef HAVE_LONG_LONG
 	long long llong_val;
 	unsigned long long ullong_val;
 #endif
@@ -1114,11 +1158,11 @@ typedef struct _pSLang_Token_Type
 	 * when reading a preparsed file, I want to keep the user-specified
 	 * form to preserve the precision.  Using a union member would mean
 	 * converting the double to a string when creating a preparsed file.
-	 * We can avoid the issues associated with this by just storing 
+	 * We can avoid the issues associated with this by just storing
 	 * floating point values as strings.
 	 */
 	SLFUTURE_CONST char *s_val;		       /* Used for IDENT_TOKEN, DOUBLE_TOKEN, etc...  */
-	
+
 	SLang_BString_Type *b_val;
 	_pSLang_Multiline_String_Type *multistring_val;
      } v;
@@ -1128,6 +1172,7 @@ typedef struct _pSLang_Token_Type
 #define SLTOKEN_IS_NEGATIVE		0x001
 #define SLTOKEN_OVERFLOW_CHECKED	0x002
 #define SLTOKEN_IS_HEX			0x004
+#define SLTOKEN_IS_BINARY		0x008
 #define SLTOKEN_TYPE_INTEGER		0x100
 #define SLTOKEN_TYPE_FLOAT		0x200
 #define SLTOKEN_TYPE_NUMBER		(SLTOKEN_TYPE_INTEGER|SLTOKEN_TYPE_FLOAT)
@@ -1142,7 +1187,7 @@ typedef struct _pSLang_Token_Type
 _pSLang_Token_Type;
 
 /* return token type or EOF_TOKEN upon error */
-extern _pSLtok_Type _pSLtoken_init_slstring_token (_pSLang_Token_Type *, _pSLtok_Type, 
+extern _pSLtok_Type _pSLtoken_init_slstring_token (_pSLang_Token_Type *, _pSLtok_Type,
 						   SLFUTURE_CONST char *, unsigned int);
 
 extern void _pSLcompile (_pSLang_Token_Type *);
@@ -1161,11 +1206,11 @@ extern int _pSLutf8_mode;
 extern int _pSLinterp_UTF8_Mode;	       /* non-zero for interpreter */
 extern int _pSLtt_UTF8_Mode;
 
-extern SLuchar_Type *_pSLinterp_decode_wchar (SLuchar_Type *u, 
-					     SLuchar_Type *umax, 
+extern SLuchar_Type *_pSLinterp_decode_wchar (SLuchar_Type *u,
+					     SLuchar_Type *umax,
 					     SLwchar_Type *chp);
 
-extern SLuchar_Type *_pSLinterp_encode_wchar (SLwchar_Type wch, 
+extern SLuchar_Type *_pSLinterp_encode_wchar (SLwchar_Type wch,
 					     SLuchar_Type *buf,
 					     unsigned int *encoded_lenp);
 
@@ -1351,8 +1396,8 @@ extern SLuchar_Type *_pSLinterp_encode_wchar (SLwchar_Type wch,
 #define BREAK_N_TOKEN	0x8b
 #define CONT_N_TOKEN	0x8c
 #define THEN_TOKEN	0x8d
-
 #define STRUCT_WITH_ASSIGN_TOKEN	0x8e
+#define FOREACH_EARGS_TOKEN	0x8f
 
 /* Note: the order here must match the order of the generic assignment tokens.
  * Also, the first token of each group must be the ?_ASSIGN_TOKEN.
@@ -1410,7 +1455,8 @@ extern SLuchar_Type *_pSLinterp_encode_wchar (SLwchar_Type wch,
 #define LAST_ASSIGN_TOKEN		0xCA
 #define IS_ASSIGN_TOKEN(t) (((t)>=FIRST_ASSIGN_TOKEN)&&((t)<=LAST_ASSIGN_TOKEN))
 
-#define _DEREF_FUNCALL_TOKEN		0xCE
+#define _DEREF_OBSOLETE_FUNCALL_TOKEN	0xCE
+#define _DEREF_FUNCALL_TOKEN		0xCF
 
 #define LOOP_THEN_TOKEN			0xD0
 /* #define LOOP_ELSE_TOKEN			0xD1 */
@@ -1581,8 +1627,8 @@ extern int _pSLang_check_signals_hook (VOID_STAR);
  *   extern fun (void **addr);
  *   double x;
  *   fun ((void **) &x);   <--- warning
- *   fun (VOID_STAR_STAR(&x)); 
- * 
+ *   fun (VOID_STAR_STAR(&x));
+ *
  * Otherwise a type-punning warning could result due to strict-aliasing.
  */
 #define VOID_STAR_STAR(addr) (VOID_STAR *)(VOID_STAR)(addr)
