@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2009 John E. Davis
+Copyright (C) 2004-2010 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -16,10 +16,9 @@ General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-USA.  
+USA.
 */
 
-
 #include "slinclud.h"
 
 #include <math.h>
@@ -138,7 +137,7 @@ SLtype _pSLarith_Arith_Types [MAX_ARITHMETIC_TYPES+1] =
 };
 
 static SLtype Alias_Map [MAX_ARITHMETIC_TYPES];
-   
+
 /* Here are a bunch of functions to convert from one type to another.  To
  * facilitate the process, macros will be used.
  */
@@ -203,12 +202,12 @@ SLarith_get_to_double_fun (SLtype type, unsigned int *sizeof_type)
 
    if ((type < SLANG_CHAR_TYPE) || (type > MAX_SLARITH_TYPE))
      return NULL;
-   
+
    t = To_Double_Fun_Table + (type - SLANG_CHAR_TYPE);
-   if ((sizeof_type != NULL) 
+   if ((sizeof_type != NULL)
        && (t->to_double_fun != NULL))
      *sizeof_type = t->sizeof_type;
-   
+
    return t->to_double_fun;
 }
 #endif				       /* SLANG_HAS_FLOAT */
@@ -250,6 +249,7 @@ SLarith_get_to_double_fun (SLtype type, unsigned int *sizeof_type)
 #define PUSH_SCALAR_OBJ_FUN(x) SLclass_push_int_obj(SLANG_UINT_TYPE,(int)(x))
 #define PUSH_POW_OBJ_FUN(x) SLclass_push_double_obj(SLANG_DOUBLE_TYPE, (x))
 #define CMP_FUNCTION uint_cmp_function
+#define TO_BINARY_FUNCTION uint_to_binary
 #include "slarith.inc"
 
 #if LONG_IS_NOT_INT
@@ -290,6 +290,7 @@ SLarith_get_to_double_fun (SLtype type, unsigned int *sizeof_type)
 #define PUSH_SCALAR_OBJ_FUN(x) SLclass_push_long_obj(SLANG_ULONG_TYPE,(long)(x))
 #define PUSH_POW_OBJ_FUN(x) SLclass_push_double_obj(SLANG_DOUBLE_TYPE, (x))
 #define CMP_FUNCTION ulong_cmp_function
+#define TO_BINARY_FUNCTION ulong_to_binary
 #include "slarith.inc"
 #else
 #define long_long_bin_op	int_int_bin_op
@@ -298,6 +299,7 @@ SLarith_get_to_double_fun (SLtype type, unsigned int *sizeof_type)
 #define ulong_unary_op		uint_unary_op
 #define long_cmp_function	int_cmp_function
 #define ulong_cmp_function	uint_cmp_function
+#define ulong_to_binary		uint_to_binary
 #endif				       /* LONG_IS_NOT_INT */
 
 #ifdef HAVE_LONG_LONG
@@ -339,12 +341,14 @@ SLarith_get_to_double_fun (SLtype type, unsigned int *sizeof_type)
 #  define PUSH_SCALAR_OBJ_FUN(x) SLclass_push_llong_obj(SLANG_ULLONG_TYPE,(long long)(x))
 #  define PUSH_POW_OBJ_FUN(x) SLclass_push_double_obj(SLANG_DOUBLE_TYPE, (x))
 #  define CMP_FUNCTION ullong_cmp_function
+#  define TO_BINARY_FUNCTION ullong_to_binary
 #  include "slarith.inc"
 # else
 #  define llong_llong_bin_op long_long_bin_op
 #  define ullong_ullong_bin_op ulong_ulong_bin_op
 #  define llong_llong_scalar_bin_op long_long_scalar_bin_op
 #  define ullong_ullong_scalar_bin_op ulong_ulong_scalar_bin_op
+#  define ullong_to_binary ulong_to_binary
 # endif				       /* LLONG_IS_NOT_LONG */
 #endif				       /* HAVE_LONG_LONG */
 
@@ -405,6 +409,7 @@ SLarith_get_to_double_fun (SLtype type, unsigned int *sizeof_type)
 #define ABS_FUNCTION(x) (x)
 #define SIGN_FUNCTION(x) (((x) > 0) ? 1 : 0)
 #define CMP_FUNCTION uchar_cmp_function
+#define TO_BINARY_FUNCTION uchar_to_binary
 #include "slarith.inc"
 
 #if SHORT_IS_NOT_INT
@@ -425,15 +430,15 @@ SLarith_get_to_double_fun (SLtype type, unsigned int *sizeof_type)
 #define ABS_FUNCTION(x) (x)
 #define SIGN_FUNCTION(x) (((x) > 0) ? 1 : 0)
 #define CMP_FUNCTION ushort_cmp_function
+#define TO_BINARY_FUNCTION ushort_to_binary
 #include "slarith.inc"
 #endif				       /* SHORT_IS_NOT_INT */
 
-
 int _pSLarith_get_precedence (SLtype type)
 {
    if ((type < SLANG_CHAR_TYPE) || (type > MAX_SLARITH_TYPE))
      return -1;
- 
+
    type = Alias_Map[TYPE_TO_TABLE_INDEX(type)];
    return type - SLANG_CHAR_TYPE;
 }
@@ -479,7 +484,7 @@ static SLtype promote_to_common_type (SLtype a, SLtype b)
    if (a == b)
      return a1;
    b = _pSLarith_promote_type (b);
-   
+
    return (a1 > b) ? a1 : b;
 }
 
@@ -526,8 +531,8 @@ static int arith_bin_op_result (int op, SLtype a_type, SLtype b_type,
 }
 
 typedef int (*Bin_Fun_Type) (int,
-			     SLtype, VOID_STAR, unsigned int,
-			     SLtype, VOID_STAR, unsigned int,
+			     SLtype, VOID_STAR, SLuindex_Type,
+			     SLtype, VOID_STAR, SLuindex_Type,
 			     VOID_STAR);
 
 /* This array of functions must be indexed by precedence after arithmetic
@@ -554,8 +559,8 @@ static Bin_Fun_Type Bin_Fun_Map [MAX_ARITHMETIC_TYPES] =
 };
 
 static int arith_bin_op (int op,
-			 SLtype a_type, VOID_STAR ap, unsigned int na,
-			 SLtype b_type, VOID_STAR bp, unsigned int nb,
+			 SLtype a_type, VOID_STAR ap, SLuindex_Type na,
+			 SLtype b_type, VOID_STAR bp, SLuindex_Type nb,
 			 VOID_STAR cp)
 {
    Bin_Fun_Type binfun;
@@ -644,7 +649,6 @@ static int arith_unary_op_result (int op, SLtype a, SLtype *b)
    return 1;
 }
 
-
 static int integer_pop (SLtype type, VOID_STAR ptr)
 {
    SLang_Object_Type obj;
@@ -776,7 +780,7 @@ int SLang_push_ulong (unsigned long i)
    return SLclass_push_long_obj (_pSLANG_ULONG_TYPE, (long) i);
 }
 
-#if HAVE_LONG_LONG
+#ifdef HAVE_LONG_LONG
 int SLang_push_long_long (long long i)
 {
    return SLclass_push_llong_obj (_pSLANG_LLONG_TYPE, i);
@@ -787,18 +791,27 @@ int SLang_push_ulong_long (unsigned long long i)
 }
 #endif
 
+int SLang_pop_strlen_type (SLstrlen_Type *ip)
+{
+   return SLang_pop_uint (ip);
+}
+int SLang_push_strlen_type (SLstrlen_Type i)
+{
+   return SLang_push_uint (i);
+}
+
 _INLINE_
-int _pSLarith_typecast (SLtype a_type, VOID_STAR ap, unsigned int na,
-		       SLtype b_type, VOID_STAR bp)
+int _pSLarith_typecast (SLtype a_type, VOID_STAR ap, SLuindex_Type na,
+			SLtype b_type, VOID_STAR bp)
 {
    int i, j;
 
-   void (*copy)(VOID_STAR, VOID_STAR, unsigned int);
+   void (*copy)(VOID_STAR, VOID_STAR, SLuindex_Type);
 
    i = TYPE_TO_TABLE_INDEX (a_type);
    j = TYPE_TO_TABLE_INDEX (b_type);
 
-   copy = (void (*)(VOID_STAR, VOID_STAR, unsigned int))
+   copy = (void (*)(VOID_STAR, VOID_STAR, SLuindex_Type))
      Binary_Matrix[i][j].copy_function;
 
    (*copy) (bp, ap, na);
@@ -835,7 +848,7 @@ int SLang_pop_double (double *x)
       case SLANG_UINT_TYPE: *x = (double) obj.v.uint_val; break;
       case SLANG_LONG_TYPE: *x = (double) obj.v.long_val; break;
       case SLANG_ULONG_TYPE: *x = (double) obj.v.ulong_val; break;
-#ifdef HAVE_LONG_LONG	
+#ifdef HAVE_LONG_LONG
       case SLANG_LLONG_TYPE: *x = (double) obj.v.llong_val; break;
       case SLANG_ULLONG_TYPE: *x = (double) obj.v.ullong_val; break;
 #endif
@@ -929,15 +942,15 @@ void _pSLset_double_format (SLCONST char *fmt)
 
    if (*s++ != '%')
      return;
-   
+
    /* 0 or more flags */
-   while ((*s == '#') || (*s == '0') || (*s == '-') 
+   while ((*s == '#') || (*s == '0') || (*s == '-')
 	  || (*s == ' ') || (*s == '+'))
      s++;
 
    /* field width */
    while (isdigit (*s)) s++;
-   
+
    /* precision */
    if (*s == '.')
      {
@@ -967,7 +980,7 @@ void _pSLset_double_format (SLCONST char *fmt)
 	Double_Format_Ptr = Double_Format;
 	return;
      }
-   
+
    if ((*s == 'S') || (*s == 's'))
      {
 	s++;
@@ -986,7 +999,7 @@ SLCONST char *_pSLget_double_format (void)
 {
    if (Double_Format_Ptr == NULL)
      return "%S";
-   
+
    return Double_Format_Ptr;
 }
 
@@ -1017,7 +1030,7 @@ static void check_decimal (char *buf, unsigned int buflen, double x)
 	if (ch != '.')
 	  return;			       /* something else */
 
-	/* We are at a decimal point.  If expon > 1, then buf does not contain 
+	/* We are at a decimal point.  If expon > 1, then buf does not contain
 	 * an exponential formatted quantity.
 	 */
 	if (count <= expon_threshold)
@@ -1031,7 +1044,7 @@ static void check_decimal (char *buf, unsigned int buflen, double x)
      }
 
    /* We get here only when *b==0. */
-   
+
    if ((has_point == 0) && (count <= 6))
      {
 	if (b + 3 >= bufmax)
@@ -1046,7 +1059,7 @@ static void check_decimal (char *buf, unsigned int buflen, double x)
      }
 
    expon = count-1;
-   
+
    /* Now add on the exponent.  First move the decimal point but drop trailing 0s */
    while ((count > 1) && (*(b-1) == '0'))
      {
@@ -1055,7 +1068,7 @@ static void check_decimal (char *buf, unsigned int buflen, double x)
      }
 
    if (count > 1)
-     {	
+     {
 	while (count > 1)
 	  {
 	     bstart[count] = bstart[count-1];
@@ -1069,7 +1082,72 @@ static void check_decimal (char *buf, unsigned int buflen, double x)
    if (EOF == SLsnprintf (b, bufmax-b, "e+%02d", expon))
      sprintf (buf, "%e", x);
 }
-	
+
+static int massage_decimal_buffer (char *inbuf, char *buf,
+				   unsigned int buflen, unsigned int min_slen)
+{
+   char *s;
+   unsigned int slen, count;
+   char c;
+
+   slen = strlen(inbuf);
+   if ((slen < min_slen) || (slen+1 > buflen))
+     return 0;
+
+   s = inbuf + slen;
+   s -= 2;			       /* skip last digit */
+   c = *s;
+   if ((c != '0') && (c != '9'))
+     return 0;
+   s--;
+
+   count = 0;
+   while ((s > inbuf) && (*s == c))
+     {
+	count++;
+	s--;
+     }
+
+   if ((count < 4) || (0 == isdigit (*s)))
+     return 0;
+
+   if (c == '9')
+     {
+	/* e.g., 9.699999999999999 */
+	slen = s-inbuf;
+	memcpy (buf, inbuf, slen);
+	buf[slen] = *s + 1;	       /* assumes ascii */
+	buf[slen+1] = 0;
+     }
+   else
+     {
+	/* 9.300000000000001 */
+	slen = (s+1)-inbuf;
+	memcpy (buf, inbuf, slen);
+	buf[slen] = 0;
+     }
+
+   return 1;
+}
+
+static void massage_double_buffer (char *inbuf, double x)
+{
+   char buf[1024];
+
+   if (massage_decimal_buffer (inbuf, buf, sizeof(buf), 16)
+       && (atof(buf) == x))
+     strcpy (inbuf, buf);
+}
+
+static void massage_float_buffer (char *inbuf, float x)
+{
+   char buf[1024];
+
+   if (massage_decimal_buffer (inbuf, buf, sizeof(buf), 8)
+       && ((float)atof(buf) == x))
+     strcpy (inbuf, buf);
+}
+
 static void default_format_double (double x, char *buf, unsigned int buflen)
 {
    if (EOF == SLsnprintf (buf, buflen, "%.16g", x))
@@ -1086,6 +1164,7 @@ static void default_format_double (double x, char *buf, unsigned int buflen)
 	     return;
 	  }
      }
+   massage_double_buffer (buf,x);
    check_decimal (buf, buflen, x);
 }
 
@@ -1104,6 +1183,7 @@ static void default_format_float (float x, char *buf, unsigned int buflen)
 	     return;
 	  }
      }
+   massage_float_buffer (buf, x);
    check_decimal (buf, buflen, x);
 }
 #endif
@@ -1218,7 +1298,7 @@ typedef struct
    SLFUTURE_CONST char *name;
    SLtype data_type;
    unsigned int sizeof_type;
-   int (*unary_fun)(int, SLtype, VOID_STAR, unsigned int, VOID_STAR);
+   int (*unary_fun)(int, SLtype, VOID_STAR, SLuindex_Type, VOID_STAR);
    int (*push_literal) (SLtype, VOID_STAR);
    void (*byte_code_destroy)(SLtype, VOID_STAR);
    int (*cmp_fun) (SLtype, VOID_STAR, VOID_STAR, int *);
@@ -1263,6 +1343,108 @@ static Integer_Info_Type Integer_Types [NUM_INTEGER_TYPES] =
 #endif
 };
 
+int _pSLformat_as_binary (unsigned int min_num_bits, int use_binary_prefix)
+{
+#ifdef HAVE_LONG_LONG
+   char buf [2*8*SIZEOF_LONG_LONG];
+#else
+   char buf [2*8*SIZEOF_LONG];
+#endif
+   char *bufp;
+   int ret;
+   unsigned int buflen;
+
+   bufp = buf;
+   buflen = sizeof(buf);
+   if (use_binary_prefix)
+     {
+	*bufp++ = '0';
+	*bufp++ = 'b';
+	buflen -= 2;
+     }
+
+   switch (SLang_peek_at_stack ())
+     {
+      default:
+      case SLANG_INT_TYPE:
+      case SLANG_UINT_TYPE:
+	  {
+	     unsigned int u;
+	     if (-1 == SLang_pop_uint (&u))
+	       return -1;
+	     ret = uint_to_binary (u, bufp, buflen, min_num_bits);
+	  }
+	break;
+
+      case SLANG_CHAR_TYPE:
+      case SLANG_UCHAR_TYPE:
+	  {
+	     unsigned char u;
+	     if (-1 == SLang_pop_uchar (&u))
+	       return -1;
+	     ret = uchar_to_binary (u, bufp, buflen, min_num_bits);
+	  }
+	break;
+
+      case SLANG_SHORT_TYPE:
+      case SLANG_USHORT_TYPE:
+	  {
+	     unsigned short u;
+	     if (-1 == SLang_pop_ushort (&u))
+	       return -1;
+	     ret = ushort_to_binary (u, bufp, buflen, min_num_bits);
+	  }
+	break;
+
+      case SLANG_LONG_TYPE:
+      case SLANG_ULONG_TYPE:
+	  {
+	     unsigned long u;
+	     if (-1 == SLang_pop_ulong (&u))
+	       return -1;
+	     ret = ulong_to_binary (u, bufp, buflen, min_num_bits);
+	  }
+	break;
+
+#ifdef HAVE_LONG_LONG
+      case SLANG_LLONG_TYPE:
+      case SLANG_ULLONG_TYPE:
+	  {
+	     unsigned long long u;
+	     if (-1 == SLang_pop_ulong_long (&u))
+	       return -1;
+	     ret = ullong_to_binary (u, bufp, buflen, min_num_bits);
+	  }
+	break;
+#endif
+     }
+   if (ret == -1)
+     {
+	SLang_verror (SL_INTERNAL_ERROR, "Buffer is not large enough for the binary representations");
+	return -1;
+     }
+
+   (void) SLang_push_string (buf);
+   return 0;
+}
+
+#if 0
+static void to_binary_string_intrin (void)
+{
+   unsigned int min_num_bits = 0;
+
+   if (SLang_Num_Function_Args == 2)
+     {
+	int n;
+	if (-1 == SLang_pop_int (&n))
+	  return;
+	if (n > 0)
+	  min_num_bits = (unsigned int) n;
+     }
+
+   (void) _pSLformat_as_binary (min_num_bits, 0);
+}
+#endif
 static int create_synonyms (void)
 {
    static SLFUTURE_CONST char *names[8] =
@@ -1345,7 +1527,7 @@ static int create_synonyms (void)
        || (-1 == _pSLclass_copy_class (SLANG_ULLONG_TYPE, _pSLANG_ULONG_TYPE)))
      return -1;
 #endif
-   
+
    return 0;
 }
 
@@ -1386,6 +1568,14 @@ static SLang_Arith_Binary_Type Binary_Table [] =
    SLANG_END_ARITH_BINARY_TABLE
 };
 
+static SLang_Intrin_Fun_Type Intrinsic_Table [] =
+{
+#if 0				       /* need to think of a better name */
+   MAKE_INTRINSIC_0("to_binary_string", to_binary_string_intrin, SLANG_VOID_TYPE),
+#endif
+   SLANG_END_INTRIN_FUN_TABLE
+};
+
 static SLang_IConstant_Type IConst_Table [] =
 {
 #if defined(SHRT_MIN) && defined(SHRT_MAX)
@@ -1402,6 +1592,11 @@ static SLang_IConstant_Type IConst_Table [] =
 #if defined(UINT_MAX)
    MAKE_ICONSTANT_T("UINT_MAX", UINT_MAX, SLANG_UINT_TYPE),
 #endif
+   SLANG_END_ICONST_TABLE
+};
+
+static SLang_LConstant_Type LConst_Table [] =
+{
 #if defined(LONG_MIN) && defined(LONG_MAX)
    MAKE_LCONSTANT_T("LONG_MIN", LONG_MIN, _pSLANG_LONG_TYPE),
    MAKE_LCONSTANT_T("LONG_MAX", LONG_MAX, _pSLANG_LONG_TYPE),
@@ -1409,20 +1604,30 @@ static SLang_IConstant_Type IConst_Table [] =
 #if defined(ULONG_MAX)
    MAKE_LCONSTANT_T("ULONG_MAX", ULONG_MAX, _pSLANG_ULONG_TYPE),
 #endif
-   SLANG_END_ICONST_TABLE
+   SLANG_END_LCONST_TABLE
 };
 
 #ifdef HAVE_LONG_LONG
-static SLang_LLConstant_Type LLConst_Table[] =
+# ifndef LLONG_MAX
+#  if (SIZEOF_LONG_LONG == 8)
+#   define LLONG_MAX 9223372036854775807LL
+/* C90 does not have positive constants-- only negated negative ones.  Hence,
+ * LLONG_MIN is -9223372036854775808LL, but 9223372036854775808LL is too big.
+ */
+#   define LLONG_MIN (-LLONG_MAX - 1LL)
+#   define ULLONG_MAX 18446744073709551615ULL
+#  endif
+# endif
+static _pSLang_LLConstant_Type LLConst_Table[] =
 {
 #if defined(LLONG_MIN) && defined(LLONG_MAX)
-   MAKE_LLCONSTANT("LLONG_MIN", LLONG_MIN),
-   MAKE_LLCONSTANT("LLONG_MAX", LLONG_MAX),
+   _pMAKE_LLCONSTANT_T("LLONG_MIN", LLONG_MIN, SLANG_LLONG_TYPE),
+   _pMAKE_LLCONSTANT_T("LLONG_MAX", LLONG_MAX, SLANG_LLONG_TYPE),
 #endif
 #if defined(ULLONG_MAX)
-   MAKE_LLCONSTANT_T("ULLONG_MAX", ULLONG_MAX, SLANG_ULLONG_TYPE),
+   _pMAKE_LLCONSTANT_T("ULLONG_MAX", ULLONG_MAX, SLANG_ULLONG_TYPE),
 #endif
-   SLANG_END_LLCONST_TABLE
+   _pSLANG_END_LLCONST_TABLE
 };
 #endif
 
@@ -1484,7 +1689,6 @@ static void compute_inf_an_nan (void)
 #endif
 }
 
-
 int _pSLarith_register_types (void)
 {
    SLang_Class_Type *cl;
@@ -1493,7 +1697,7 @@ int _pSLarith_register_types (void)
 
 #if defined(HAVE_SETLOCALE) && defined(LC_NUMERIC)
    /* make sure decimal point it used --- the parser requires it */
-   (void) setlocale (LC_NUMERIC, "C"); 
+   (void) setlocale (LC_NUMERIC, "C");
 #endif
 
    for (i = 0; i < NUM_INTEGER_TYPES; i++)
@@ -1586,7 +1790,7 @@ int _pSLarith_register_types (void)
 	if (Alias_Map[TYPE_TO_TABLE_INDEX(a_type)] != a_type)
 	  continue;
 #endif
-	if (a_type == 0) 
+	if (a_type == 0)
 	  continue;
 
 	for (j = 0; j < MAX_ARITHMETIC_TYPES; j++)
@@ -1597,7 +1801,7 @@ int _pSLarith_register_types (void)
 	     if (b_type == 0)
 	       continue;
 	     /* Allow implicit typecast, except from int to float */
-	     implicit_ok = ((b_type >= SLANG_FLOAT_TYPE) 
+	     implicit_ok = ((b_type >= SLANG_FLOAT_TYPE)
 			    || (a_type < SLANG_FLOAT_TYPE));
 
 	     if (-1 == SLclass_add_binary_op (a_type, b_type, arith_bin_op, arith_bin_op_result))
@@ -1609,18 +1813,21 @@ int _pSLarith_register_types (void)
 	  }
      }
 
+   if (-1 == SLadd_intrin_fun_table (Intrinsic_Table, NULL))
+     return -1;
    if (-1 == _pSLadd_arith_unary_table (Unary_Table, NULL))
      return -1;
    if (-1 == _pSLadd_arith_binary_table (Binary_Table, NULL))
      return -1;
 
    if ((-1 == SLadd_iconstant_table (IConst_Table, NULL))
+       || (-1 == SLadd_lconstant_table (LConst_Table, NULL))
 #if SLANG_HAS_FLOAT
        || (-1 == SLadd_fconstant_table (FConst_Table, NULL))
        || (-1 == SLadd_dconstant_table (DConst_Table, NULL))
 #endif
-#if HAVE_LONG_LONG
-       || (-1 == SLadd_llconstant_table (LLConst_Table, NULL))
+#ifdef HAVE_LONG_LONG
+       || (-1 == _pSLadd_llconstant_table (LLConst_Table, NULL))
 #endif
        )
      return -1;
@@ -1641,7 +1848,7 @@ static void promote_objs (SLang_Object_Type *a, SLang_Object_Type *b,
 
    ia = a->o_data_type;
    ib = b->o_data_type;
-   
+
    ic = _pSLarith_promote_type (ia);
 
    if (ic == ib) id = ic;	       /* already promoted */
@@ -1695,12 +1902,11 @@ int _pSLarith_bin_op (SLang_Object_Type *oa, SLang_Object_Type *ob, int op)
 	promote_objs (oa, ob, &obj_a, &obj_b);
 	oa = &obj_a;
 	ob = &obj_b;
-	
+
 	a_type = oa->o_data_type;
 	/* b_type = ob->data_type; */
      }
-   
-	  
+
    switch (a_type)
      {
       case SLANG_CHAR_TYPE:
@@ -1730,7 +1936,7 @@ int _pSLarith_bin_op (SLang_Object_Type *oa, SLang_Object_Type *ob, int op)
 #endif
       case SLANG_UINT_TYPE:
 	return uint_uint_scalar_bin_op (oa->v.uint_val, ob->v.uint_val, op);
-	
+
 #if LONG_IS_NOT_INT
       case SLANG_LONG_TYPE:
 	return long_long_scalar_bin_op (oa->v.long_val, ob->v.long_val, op);
