@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2009 John E. Davis
+Copyright (C) 2004-2010 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -16,7 +16,7 @@ General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-USA.  
+USA.
 */
 
 #include "slinclud.h"
@@ -301,8 +301,8 @@ static void free_n_bstrings (SLang_BString_Type **a, unsigned int n)
 
 static int
 bstring_bstring_bin_op (int op,
-			SLtype a_type, VOID_STAR ap, unsigned int na,
-			SLtype b_type, VOID_STAR bp, unsigned int nb,
+			SLtype a_type, VOID_STAR ap, SLuindex_Type na,
+			SLtype b_type, VOID_STAR bp, SLuindex_Type nb,
 			VOID_STAR cp)
 {
    char *ic;
@@ -455,8 +455,8 @@ make_n_bstrings (SLang_BString_Type **b, char **a, unsigned int n, int ptr_type)
 
 static int
 bstring_string_bin_op (int op,
-		       SLtype a_type, VOID_STAR ap, unsigned int na,
-		       SLtype b_type, VOID_STAR bp, unsigned int nb,
+		       SLtype a_type, VOID_STAR ap, SLuindex_Type na,
+		       SLtype b_type, VOID_STAR bp, SLuindex_Type nb,
 		       VOID_STAR cp)
 {
    SLang_BString_Type **b;
@@ -477,8 +477,8 @@ bstring_string_bin_op (int op,
 
 static int
 string_bstring_bin_op (int op,
-		       SLtype a_type, VOID_STAR ap, unsigned int na,
-		       SLtype b_type, VOID_STAR bp, unsigned int nb,
+		       SLtype a_type, VOID_STAR ap, SLuindex_Type na,
+		       SLtype b_type, VOID_STAR bp, SLuindex_Type nb,
 		       VOID_STAR cp)
 {
    SLang_BString_Type **a;
@@ -511,7 +511,7 @@ static int bstring_push (SLtype unused, VOID_STAR sptr)
    return SLang_push_bstring (*(SLang_BString_Type **) sptr);
 }
 
-static int string_to_bstring (SLtype a_type, VOID_STAR ap, unsigned int na,
+static int string_to_bstring (SLtype a_type, VOID_STAR ap, SLuindex_Type na,
 			      SLtype b_type, VOID_STAR bp)
 {
    char **s;
@@ -529,11 +529,11 @@ static int string_to_bstring (SLtype a_type, VOID_STAR ap, unsigned int na,
    return 1;
 }
 
-static int bstring_to_string (SLtype a_type, VOID_STAR ap, unsigned int na,
+static int bstring_to_string (SLtype a_type, VOID_STAR ap, SLuindex_Type na,
 			      SLtype b_type, VOID_STAR bp)
 {
    char **s;
-   unsigned int i;
+   SLuindex_Type i;
    SLang_BString_Type **a;
 
    (void) a_type;
@@ -630,7 +630,7 @@ static unsigned int count_byte_occurances (SLang_BString_Type *b, unsigned char
 
    bytes = BS_GET_POINTER(b);
    bytes_max = bytes + b->len;
-   
+
    n = 0;
    while (bytes < bytes_max)
      {
@@ -640,6 +640,49 @@ static unsigned int count_byte_occurances (SLang_BString_Type *b, unsigned char
    return n;
 }
 
+/* returns the character position of substring in a string or null */
+static int issubbytes (SLang_BString_Type *as, SLang_BString_Type *bs)
+{
+   unsigned int lena, lenb;
+   unsigned char *a, *b, *amax, *bmax, *astart;
+   unsigned char ch0;
+
+   a = BS_GET_POINTER(as);
+   b = BS_GET_POINTER(bs);
+   lena = as->len;
+   lenb = bs->len;
+
+   if ((lenb > lena) || (lenb == 0))
+     return 0;
+
+   lena -= lenb;
+   amax = a + lena;
+   bmax = b + lenb;
+
+   astart = a;
+   ch0 = *b++;
+   while (a <= amax)
+     {
+	unsigned char *a0, *b0;
+
+	if (ch0 != *a++)
+	  continue;
+	a0 = a;
+	b0 = b;
+	while ((b < bmax) && (*a == *b))
+	  {
+	     a++;
+	     b++;
+	  }
+	if (b == bmax)
+	  return (a0 - astart);
+
+	a = a0;
+	b = b0;
+     }
+   return 0;
+}
+
 static SLang_Intrin_Fun_Type BString_Table [] = /*{{{*/
 {
    MAKE_INTRINSIC_1("bstrlen",  bstrlen_cmd, SLANG_UINT_TYPE, SLANG_BSTRING_TYPE),
@@ -648,10 +691,10 @@ static SLang_Intrin_Fun_Type BString_Table [] = /*{{{*/
    MAKE_INTRINSIC_2("unpack", _pSLunpack, SLANG_VOID_TYPE, SLANG_STRING_TYPE, SLANG_BSTRING_TYPE),
    MAKE_INTRINSIC_1("pad_pack_format", _pSLpack_pad_format, SLANG_VOID_TYPE, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_1("sizeof_pack", _pSLpack_compute_size, SLANG_UINT_TYPE, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_2("is_substrbytes", issubbytes, SLANG_INT_TYPE, SLANG_BSTRING_TYPE, SLANG_BSTRING_TYPE),
    SLANG_END_INTRIN_FUN_TABLE
 };
 
-
 struct _pSLang_Foreach_Context_Type
 {
    SLang_BString_Type *bstr;
@@ -703,7 +746,7 @@ _pSLbstring_foreach_open (SLtype type, unsigned int num)
 		      "'foreach ([B]String_Type) using' requires single control value (chars|bytes)");
 	return NULL;
      }
-   
+
    /* In UTF-8 mode, chars and bytes are synonymous */
    if (_pSLinterp_UTF8_Mode == 0)
      using_chars = 0;
@@ -716,7 +759,7 @@ _pSLbstring_foreach_open (SLtype type, unsigned int num)
      }
 
    memset ((char *) c, 0, sizeof (SLang_Foreach_Context_Type));
-   
+
    c->bstr = bstr;
    c->s = SLbstring_get_pointer (bstr, &len);
    c->smax = c->s + len;
@@ -737,7 +780,7 @@ int _pSLbstring_foreach (SLtype type, SLang_Foreach_Context_Type *c)
    unsigned char ch;
    SLwchar_Type wch;
    unsigned char *s, *s1, *smax;
-   
+
    (void) type;
 
    s = c->s;
@@ -763,13 +806,13 @@ int _pSLbstring_foreach (SLtype type, SLang_Foreach_Context_Type *c)
 	/* Invalid encoded char-- return it as a negative int */
 	if (-1 == SLang_push_int (-iwch))
 	  return -1;
-	
+
 	return 1;
      }
    c->s = s1;
    if (-1 == SLang_push_wchar (wch))
      return -1;
-   
+
    return 1;
 }
 
@@ -804,7 +847,7 @@ int _pSLang_init_bstring (void)
    cl->cl_foreach_open = _pSLbstring_foreach_open;
    cl->cl_foreach_close = _pSLbstring_foreach_close;
    cl->cl_foreach = _pSLbstring_foreach;
-   
+
    if (-1 == SLadd_intrin_fun_table (BString_Table, NULL))
      return -1;
 
