@@ -1,6 +1,6 @@
 /* Complex Data Type definition for S-Lang */
 /*
-Copyright (C) 2004-2009 John E. Davis
+Copyright (C) 2004-2010 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -17,7 +17,7 @@ General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-USA.  
+USA.
 */
 
 #include "slinclud.h"
@@ -170,7 +170,6 @@ double SLcomplex_abs (double *z)
    return SLmath_hypot (z[0], z[1]);
 }
 
-
 /* It appears that FORTRAN assumes that the branch cut for the log function
  * is along the -x axis.  So, use this for atan2:
  */
@@ -436,14 +435,14 @@ static double *not_implemented (char *fun, double *p)
 double *SLcomplex_asinh (double *asinhz, double *z)
 {
    double iz[2];
-   
+
    iz[0] = -z[1];
    iz[1] = z[0];
-   
+
    (void) SLcomplex_asin (iz, iz);
    asinhz[0] = iz[1];
    asinhz[1] = -iz[0];
-   
+
    return asinhz;
 }
 
@@ -451,7 +450,7 @@ double *SLcomplex_asinh (double *asinhz, double *z)
 double *SLcomplex_acosh (double *acoshz, double *z)
 {
    double iz[2];
-   
+
    (void) SLcomplex_acos (iz, z);
    acoshz[0] = -iz[1];
    acoshz[1] = iz[0];
@@ -463,14 +462,14 @@ double *SLcomplex_acosh (double *acoshz, double *z)
 double *SLcomplex_atanh (double *atanhz, double *z)
 {
    double iz[2];
-   
+
    iz[0] = -z[1];
    iz[1] = z[0];
-   
+
    (void) SLcomplex_atan (iz, iz);
    atanhz[0] = iz[1];
    atanhz[1] = -iz[0];
-   
+
    return atanhz;
 }
 
@@ -499,14 +498,14 @@ static int complex_binary_result (int op, SLtype a, SLtype b,
 }
 
 static int complex_complex_binary (int op,
-				   SLtype a_type, VOID_STAR ap, unsigned int na,
-				   SLtype b_type, VOID_STAR bp, unsigned int nb,
+				   SLtype a_type, VOID_STAR ap, SLuindex_Type na,
+				   SLtype b_type, VOID_STAR bp, SLuindex_Type nb,
 				   VOID_STAR cp)
 {
    char *ic;
    double *a, *b, *c;
-   unsigned int n, n_max;
-   unsigned int da, db;
+   SLuindex_Type n, n_max;
+   SLuindex_Type da, db;
 
    (void) a_type;
    (void) b_type;
@@ -598,14 +597,14 @@ static int complex_complex_binary (int op,
 }
 
 static int complex_double_binary (int op,
-				  SLtype a_type, VOID_STAR ap, unsigned int na,
-				  SLtype b_type, VOID_STAR bp, unsigned int nb,
+				  SLtype a_type, VOID_STAR ap, SLuindex_Type na,
+				  SLtype b_type, VOID_STAR bp, SLuindex_Type nb,
 				  VOID_STAR cp)
 {
    char *ic;
    double *a, *b, *c;
-   unsigned int n, n_max;
-   unsigned int da, db;
+   SLuindex_Type n, n_max;
+   SLuindex_Type da, db;
 
    (void) a_type;
    (void) b_type;
@@ -700,14 +699,14 @@ static int complex_double_binary (int op,
 }
 
 static int double_complex_binary (int op,
-				  SLtype a_type, VOID_STAR ap, unsigned int na,
-				  SLtype b_type, VOID_STAR bp, unsigned int nb,
+				  SLtype a_type, VOID_STAR ap, SLuindex_Type na,
+				  SLtype b_type, VOID_STAR bp, SLuindex_Type nb,
 				  VOID_STAR cp)
 {
    char *ic;
    double *a, *b, *c;
-   unsigned int n, n_max;
-   unsigned int da, db;
+   SLuindex_Type n, n_max;
+   SLuindex_Type da, db;
 
    (void) a_type;
    (void) b_type;
@@ -803,15 +802,15 @@ static int double_complex_binary (int op,
 }
 
 static int complex_generic_binary (int op,
-				   SLtype a_type, VOID_STAR ap, unsigned int na,
-				   SLtype b_type, VOID_STAR bp, unsigned int nb,
+				   SLtype a_type, VOID_STAR ap, SLuindex_Type na,
+				   SLtype b_type, VOID_STAR bp, SLuindex_Type nb,
 				   VOID_STAR cp)
 {
    char *ic;
    char *b;
    double *a, *c;
-   unsigned int n, n_max;
-   unsigned int da, db;
+   SLuindex_Type n, n_max;
+   SLuindex_Type da, db;
    unsigned int sizeof_b;
    SLang_To_Double_Fun_Type to_double;
 
@@ -910,14 +909,14 @@ static int complex_generic_binary (int op,
 }
 
 static int generic_complex_binary (int op,
-				   SLtype a_type, VOID_STAR ap, unsigned int na,
-				   SLtype b_type, VOID_STAR bp, unsigned int nb,
+				   SLtype a_type, VOID_STAR ap, SLuindex_Type na,
+				   SLtype b_type, VOID_STAR bp, SLuindex_Type nb,
 				   VOID_STAR cp)
 {
    double *b, *c;
    char *a, *ic;
-   unsigned int n, n_max;
-   unsigned int da, db;
+   SLuindex_Type n, n_max;
+   SLuindex_Type da, db;
    unsigned int sizeof_a;
    SLang_To_Double_Fun_Type to_double;
 
@@ -1044,10 +1043,10 @@ static int complex_unary_result (int op, SLtype a, SLtype *b)
 }
 
 static int complex_unary (int op,
-			  SLtype a_type, VOID_STAR ap, unsigned int na,
+			  SLtype a_type, VOID_STAR ap, SLuindex_Type na,
 			  VOID_STAR bp)
 {
-   unsigned int n;
+   SLuindex_Type n;
    double *a, *b;
    int *ic;
 
@@ -1112,13 +1111,13 @@ static int complex_unary (int op,
 }
 
 static int
-complex_typecast (SLtype from_type, VOID_STAR from, unsigned int num,
+complex_typecast (SLtype from_type, VOID_STAR from, SLuindex_Type num,
 		  SLtype to_type, VOID_STAR to)
 {
    double *z;
    double *d;
    char *i;
-   unsigned int n;
+   SLuindex_Type n;
    unsigned int sizeof_i;
    SLang_To_Double_Fun_Type to_double;
 
