@@ -1,6 +1,6 @@
 /* sin, cos, etc, for S-Lang */
 /*
-Copyright (C) 2004-2009 John E. Davis
+Copyright (C) 2004-2010 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -17,7 +17,7 @@ General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-USA.  
+USA.
 */
 
 #include "slinclud.h"
@@ -153,7 +153,7 @@ int _pSLmath_isinf (double x)
 static double my_round (double x)
 {
    double xf, xi;
-   
+
    xf = modf (x, &xi);		       /* x = xi + xf */
    if (xi > 0)
      {
@@ -172,7 +172,7 @@ static double my_round (double x)
      }
    else if (xf >= 0.5)		       /* xi=0 */
      return 1.0;
-	  
+
    return xi;
 }
 #endif
@@ -212,7 +212,7 @@ double _pSLmath_expm1 (double x)
    /* f = exp(x)-1
     *    = (exp(x)-1) + x - x
     *    = (exp(x)-(1+x)) + x
-    * 
+    *
     * While the above works, it is not as accurate as the algroithm below,
     * which is due to Kahan (yes, that Kahan).
     */
@@ -239,12 +239,12 @@ double _pSLmath_expm1 (double x)
 static double my_asinh (double x)
 {
    /* f(x) = log(x + sqrt(x^2+1))
-    * Note: f(x)+f(-x) 
+    * Note: f(x)+f(-x)
     *          = log(x+sqrt(x^2+1))+log(-x+sqrt(x^2+1))
     *          = log(x^2+1 - x^2)
     *          = 0
     * Thus, f(-x)=-f(x) as required.
-    * 
+    *
     * So consider x>=0.
     * For large x
     *   = log(2x + sqrt(x^2+1)-x)
@@ -267,7 +267,7 @@ static double my_asinh (double x)
 
    if (x > 1.0)
      return s*log (2.0*x + 1.0/x/(1.0+sqrt(1.0+1.0/x2)));
-   
+
    return LOG1P_FUNC(x+x2/(1.0+sqrt(1.0+x2)));
 }
 #endif
@@ -294,7 +294,7 @@ static double my_acosh (double x)
 static double my_atanh (double x)
 {
    /* 0.5 * log ((1.0 + x)/(1.0 - x));  (0 <= x^2 < 1)
-    * = 0.5*log((1-x+2x)/(1-x)) 
+    * = 0.5*log((1-x+2x)/(1-x))
     * = 0.5*log(1 + 2x/(1-x))
     */
    return 0.5 * LOG1P_FUNC(2.0*x/(1-x));
@@ -302,7 +302,7 @@ static double my_atanh (double x)
 #endif
 
 static int double_math_op (int op,
-			   SLtype type, VOID_STAR ap, unsigned int na,
+			   SLtype type, VOID_STAR ap, SLuindex_Type na,
 			   VOID_STAR bp)
 {
    double *a, *b;
@@ -376,7 +376,7 @@ static int double_math_op (int op,
 	for (i = 0; i < na; i++)
 	  b[i] = 0.0;
 	return 1;
-	
+
       case SLMATH_ISINF:
 	c = (char *) bp;
 	for (i = 0; i < na; i++)
@@ -388,7 +388,7 @@ static int double_math_op (int op,
 	for (i = 0; i < na; i++)
 	  c[i] = (char) ISNAN_FUN(a[i]);
 	return 1;
-	
+
       case SLMATH_FLOOR:
 	for (i = 0; i < na; i++) b[i] = floor(a[i]);
 	break;
@@ -396,11 +396,11 @@ static int double_math_op (int op,
       case SLMATH_CEIL:
 	for (i = 0; i < na; i++) b[i] = ceil(a[i]);
 	break;
-	
+
       case SLMATH_ROUND:
 	for (i = 0; i < na; i++) b[i] = ROUND_FUN(a[i]);
 	break;
-	
+
       case SLMATH_EXPM1:
 	for (i = 0; i < na; i++) b[i] = EXPM1_FUNC(a[i]);
 	break;
@@ -409,12 +409,12 @@ static int double_math_op (int op,
 	for (i = 0; i < na; i++) b[i] = LOG1P_FUNC(a[i]);
 	break;
      }
-   
+
    return 1;
 }
 
 static int float_math_op (int op,
-			  SLtype type, VOID_STAR ap, unsigned int na,
+			  SLtype type, VOID_STAR ap, SLuindex_Type na,
 			  VOID_STAR bp)
 {
    float *a, *b;
@@ -425,7 +425,6 @@ static int float_math_op (int op,
    a = (float *) ap;
    b = (float *) bp;
 
-
    switch (op)
      {
       default:
@@ -527,7 +526,7 @@ static int float_math_op (int op,
 }
 
 static int generic_math_op (int op,
-			    SLtype type, VOID_STAR ap, unsigned int na,
+			    SLtype type, VOID_STAR ap, SLuindex_Type na,
 			    VOID_STAR bp)
 {
    double *b;
@@ -548,91 +547,91 @@ static int generic_math_op (int op,
 	return 0;
 
       case SLMATH_SINH:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = sinh (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
       case SLMATH_COSH:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = cosh (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
       case SLMATH_TANH:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = tanh (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
       case SLMATH_TAN:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = tan (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
       case SLMATH_ASIN:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = asin (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
       case SLMATH_ACOS:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = acos (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
       case SLMATH_ATAN:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = atan (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
       case SLMATH_EXP:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = exp (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
       case SLMATH_LOG:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = log (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
       case SLMATH_LOG10:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = log10 (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
       case SLMATH_SQRT:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = sqrt (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
       case SLMATH_SIN:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = sin (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
       case SLMATH_COS:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = cos (to_double ((VOID_STAR) a));
 	     a += da;
@@ -640,28 +639,27 @@ static int generic_math_op (int op,
 	break;
 
       case SLMATH_ASINH:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = ASINH_FUN (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
       case SLMATH_ATANH:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = ATANH_FUN (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
       case SLMATH_ACOSH:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = ACOSH_FUN (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
 
-
       case SLMATH_CONJ:
       case SLMATH_REAL:
 	for (i = 0; i < na; i++)
@@ -693,21 +691,21 @@ static int generic_math_op (int op,
 	  }
 	return 1;
       case SLMATH_FLOOR:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = floor (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
       case SLMATH_CEIL:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = ceil (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
       case SLMATH_ROUND:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = ROUND_FUN (to_double ((VOID_STAR) a));
 	     a += da;
@@ -715,7 +713,7 @@ static int generic_math_op (int op,
 	break;
 
       case SLMATH_EXPM1:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = EXPM1_FUNC (to_double ((VOID_STAR) a));
 	     a += da;
@@ -723,14 +721,14 @@ static int generic_math_op (int op,
 	break;
 
       case SLMATH_LOG1P:
-	for (i = 0; i < na; i++) 
+	for (i = 0; i < na; i++)
 	  {
 	     b[i] = LOG1P_FUNC (to_double ((VOID_STAR) a));
 	     a += da;
 	  }
 	break;
      }
-   
+
    return 1;
 }
 
@@ -758,7 +756,7 @@ static int complex_math_op_result (int op, SLtype a, SLtype *b)
 }
 
 static int complex_math_op (int op,
-			    SLtype type, VOID_STAR ap, unsigned int na,
+			    SLtype type, VOID_STAR ap, SLuindex_Type na,
 			    VOID_STAR bp)
 {
    double *a, *b;
@@ -887,7 +885,6 @@ typedef struct
 }
 Array_Or_Scalar_Type;
 
-
 static void free_array_or_scalar (Array_Or_Scalar_Type *ast)
 {
    if (ast->at != NULL)
@@ -968,7 +965,6 @@ static int pop_2_arrays_or_scalar (Array_Or_Scalar_Type *a, Array_Or_Scalar_Type
    return 0;
 }
 
-
 static SLang_Array_Type *create_from_tmp_array (SLang_Array_Type *a, SLang_Array_Type *b, SLtype type)
 {
    SLang_Array_Type *c;
@@ -984,11 +980,11 @@ static SLang_Array_Type *create_from_tmp_array (SLang_Array_Type *a, SLang_Array
 	return b;
      }
 
-   if (a != NULL) 
+   if (a != NULL)
      c = a;
-   else 
+   else
      c = b;
-   
+
    return SLang_create_array1 (type, 0, NULL, c->dims, c->num_dims, 1);
 }
 
@@ -1124,7 +1120,7 @@ static int do_binary_function (double (*f)(double, double))
      (void) do_df_fun (f, &a_ast, &b_ast, &c_ast);
    else
      (void) do_dd_fun (f, &a_ast, &b_ast, &c_ast);
-   
+
    free_array_or_scalar (&a_ast);
    free_array_or_scalar (&b_ast);
 
@@ -1137,16 +1133,37 @@ static int do_binary_function (double (*f)(double, double))
    return SLang_push_double (c_ast.d);
 }
 
+/* num is assumed to be at least 2 here */
+static int do_binary_function_on_nargs (double (*f)(double, double), int num)
+{
+   int depth = SLstack_depth () - num;
+
+   num--;			       /* first time consumes 2 args */
+   while (num > 0)
+     {
+	if (-1 == do_binary_function (f))
+	  {
+	     num = SLstack_depth () - depth;
+	     if (num > 0)
+	       (void) SLdo_pop_n ((unsigned int) num);
+	     return -1;
+	  }
+	num--;
+     }
+   return 0;
+}
+
 static void hypot_fun (void)
 {
    Array_Or_Scalar_Type ast;
    unsigned int num;
 
-   if (SLang_Num_Function_Args == 2)
+   if (SLang_Num_Function_Args >= 2)
      {
-	(void) do_binary_function (SLmath_hypot);
+	(void) do_binary_function_on_nargs (SLmath_hypot, SLang_Num_Function_Args);
 	return;
      }
+
    if (-1 == pop_array_or_scalar (&ast))
      return;
 
@@ -1161,52 +1178,86 @@ static void hypot_fun (void)
      {
 	float *f = ast.fptr;
 	double sum, esum, max;
-	unsigned int i;
+	unsigned int i, imax;
 
 	max = fabs((double)*f);
-	
+	imax = 0;
 	for (i = 1; i < num; i++)
 	  {
 	     double max1 = fabs((double)f[i]);
-	     if (max1 > max) max  = max1;
+	     if (max1 > max)
+	       {
+		  imax = i;
+		  max  = max1;
+	       }
 	  }
 	sum = 0.0; esum = 0.0;
-	if (max > 0.0) for (i = 0; i < num; i++)
+	if (max > 0.0)
 	  {
-	     double term = f[i]/max;
-	     double new_sum;
-	     
-	     term = term*term;
-	     new_sum = sum + term;
-	     esum += term - (new_sum-sum);
-	     sum = new_sum;
+	     for (i = 0; i < imax; i++)
+	       {
+		  double term = f[i]/max;
+		  double new_sum;
+
+		  term = term*term - esum;
+		  new_sum = sum + term;
+		  esum = (new_sum - sum) - term;
+		  sum = new_sum;
+	       }
+	     for (i = imax+1; i < num; i++)
+	       {
+		  double term = f[i]/max;
+		  double new_sum;
+
+		  term = term*term - esum;
+		  new_sum = sum + term;
+		  esum = (new_sum - sum) - term;
+		  sum = new_sum;
+	       }
 	  }
-	(void) SLang_push_float ((float) max*sqrt(sum+esum));
+	(void) SLang_push_float ((float) max*sqrt(1.0+sum));
      }
    else
      {
-	unsigned int i;
+	unsigned int i, imax;
 	double *d = ast.dptr;
 	double sum, esum, max;
 	max = fabs(*d);
-	
+	imax = 0;
 	for (i = 1; i < num; i++)
 	  {
 	     double max1 = fabs(d[i]);
-	     if (max1 > max) max  = max1;
+	     if (max1 > max)
+	       {
+		  max  = max1;
+		  imax = i;
+	       }
 	  }
 	sum = 0.0; esum = 0.0;
-	if (max > 0.0) for (i = 0; i < num; i++)
+	if (max > 0.0)
 	  {
-	     double term = d[i]/max;
-	     double new_sum;
-	     
-	     term = term*term;
-	     new_sum = sum + term;
-	     esum += term - (new_sum-sum);
-	     sum = new_sum;
+	     for (i = 0; i < imax; i++)
+	       {
+		  double term = d[i]/max;
+		  double new_sum;
+
+		  term = term*term - esum;
+		  new_sum = sum + term;
+		  esum = (new_sum - sum) - term;
+		  sum = new_sum;
+	       }
+	     for (i = imax+1; i < num; i++)
+	       {
+		  double term = d[i]/max;
+		  double new_sum;
+
+		  term = term*term - esum;
+		  new_sum = sum + term;
+		  esum = (new_sum - sum) - term;
+		  sum = new_sum;
+	       }
 	  }
-	(void) SLang_push_double (max*sqrt(sum+esum));
+	(void) SLang_push_double (max*sqrt(1.0+sum));
      }
    free_array_or_scalar (&ast);
 }
@@ -1229,7 +1280,7 @@ static void math_poly (void)
 	/* drop */
       case 2:
 	break;
-	
+
       default:
 	SLang_verror (SL_Usage_Error, "\
 Usage: y = polynom([a0,a1,...], x [,use_factorial_form])");
@@ -1270,7 +1321,7 @@ Usage: y = polynom([a0,a1,...], x [,use_factorial_form])");
 	     k--;
 	     y = a[k] + x*y;
 	  }
-	
+
 	if (ast.is_float)
 	  (void) SLang_push_float ((float) y);
 	else
@@ -1278,7 +1329,7 @@ Usage: y = polynom([a0,a1,...], x [,use_factorial_form])");
 
 	goto free_and_return;
      }
-   
+
    if (NULL == (y_at = create_from_tmp_array (ast.at, NULL, ast.at->data_type)))
      goto free_and_return;
 
@@ -1336,7 +1387,7 @@ Usage: y = polynom([a0,a1,...], x [,use_factorial_form])");
 	     yd[i] = y;
 	  }
      }
-   
+
    (void) SLang_push_array (y_at, 1);
    /* drop */
 free_and_return:
@@ -1365,14 +1416,35 @@ static double do_diff (double a, double b)
 {
    return fabs(a-b);
 }
+
 static void min_fun (void)
 {
-   (void) do_binary_function (do_min);
+   int num = SLang_Num_Function_Args;
+
+   if (num <= 0)
+     {
+	SLang_verror (SL_Usage_Error, "_min: Expecting at least one argument");
+	return;
+     }
+
+   if (num > 1)
+     (void) do_binary_function_on_nargs (do_min, num);
 }
+
 static void max_fun (void)
 {
-   (void) do_binary_function (do_max);
+   int num = SLang_Num_Function_Args;
+
+   if (num <= 0)
+     {
+	SLang_verror (SL_Usage_Error, "_max: Expecting at least one argument");
+	return;
+     }
+
+   if (num > 1)
+     (void) do_binary_function_on_nargs (do_max, num);
 }
+
 static void diff_fun (void)
 {
    (void) do_binary_function (do_diff);
@@ -1486,7 +1558,7 @@ static int do_binary_function_c (int (*f)(double, double,VOID_STAR), VOID_STAR c
 	  c_ast.at = SLang_create_array1 (SLANG_CHAR_TYPE, 0, NULL, a_ast.at->dims, a_ast.at->num_dims, 1);
 	else
 	  c_ast.at = SLang_create_array1 (SLANG_CHAR_TYPE, 0, NULL, b_ast.at->dims, b_ast.at->num_dims, 1);
-	
+
 	if (c_ast.at == NULL)
 	  {
 	     free_array_or_scalar (&a_ast);
@@ -1519,7 +1591,7 @@ static int do_binary_function_c (int (*f)(double, double,VOID_STAR), VOID_STAR c
    return SLang_push_char (c_ast.c);
 }
 
-typedef struct 
+typedef struct
 {
    double relerr;
    double abserr;
@@ -1535,13 +1607,13 @@ static int get_tolorances (int nargs, Feqs_Err_Type *ep)
 	    || (-1 == SLang_pop_double (&ep->relerr)))
 	  return -1;
 	break;
-	
+
       case 1:
 	if (-1 == SLang_pop_double (&ep->relerr))
 	  return -1;
 	ep->abserr = 0.0;
 	break;
-	
+
       default:
 	ep->relerr = 0.01;
 	ep->abserr = 1e-6;
@@ -1549,17 +1621,17 @@ static int get_tolorances (int nargs, Feqs_Err_Type *ep)
      }
    return 0;
 }
-	  
+
 static int do_feqs (double a, double b, VOID_STAR cd)
-{   
+{
    Feqs_Err_Type *e = (Feqs_Err_Type *)cd;
-   
+
    if (fabs (a-b) <= e->abserr)
      return 1;
-   
+
    if (fabs(a) > fabs(b))
      return fabs ((a-b)/a) <= e->relerr;
-   
+
    return fabs ((b-a)/b) <= e->relerr;
 }
 
@@ -1583,7 +1655,7 @@ static void do_an_feqs_fun (int (*f)(double, double, VOID_STAR))
    Feqs_Err_Type e;
    if (-1 == get_tolorances (SLang_Num_Function_Args-2, &e))
      return;
-   
+
    do_binary_function_c (f, (VOID_STAR)&e);
 }
 
@@ -1608,18 +1680,17 @@ static void fneqs_fun (void)
 }
 #endif
 
-
 static int do_nint (double x)
 {
    double xf, xi;
-   
+
    xf = modf (x, &xi);		       /* x = xi + xf */
    if (x >= 0)
      {
 	if (xf >= 0.5)
 	  return xi + 1;
      }
-   else 
+   else
      {
 	if (xf <= -0.5)
 	  return xi - 1;
@@ -1632,7 +1703,7 @@ static int float_to_nint (SLang_Array_Type *at, SLang_Array_Type *bt)
    unsigned int n, i;
    int *ip;
    float *fp;
-   
+
    fp = (float *) at->data;
    ip = (int *) bt->data;
    n = at->num_elements;
@@ -1648,7 +1719,7 @@ static int double_to_nint (SLang_Array_Type *at, SLang_Array_Type *bt)
    unsigned int n, i;
    int *ip;
    double *dp;
-   
+
    dp = (double *) at->data;
    ip = (int *) bt->data;
    n = at->num_elements;
@@ -1693,7 +1764,7 @@ static void nint_intrin (void)
 	at_to_int_fun = double_to_nint;
 	break;
      }
-   
+
    if (NULL == (bt = SLang_create_array1 (SLANG_INT_TYPE, 0, NULL, at->dims, at->num_dims, 1)))
      {
 	SLang_free_array (at);
@@ -1701,7 +1772,7 @@ static void nint_intrin (void)
      }
    if (0 == (*at_to_int_fun) (at, bt))
      (void) SLang_push_array (bt, 0);
-   
+
    SLang_free_array (bt);
    SLang_free_array (at);
 }
@@ -1734,7 +1805,7 @@ static void frexp_intrin (void)
 	(void) SLang_push_float (f);
 	(void) SLang_push_int (e);
 	return;
-	
+
       default:
       case SLANG_DOUBLE_TYPE:
 	if (-1 == SLang_pop_double (&d))
@@ -1744,21 +1815,21 @@ static void frexp_intrin (void)
 	(void) SLang_push_int (e);
 	return;
      }
-   
+
    switch (SLang_peek_at_stack1 ())
      {
       case SLANG_FLOAT_TYPE:
 	if (-1 == SLang_pop_array_of_type (&at, SLANG_FLOAT_TYPE))
 	  return;
 	break;
-	
+
       default:
       case SLANG_DOUBLE_TYPE:
 	if (-1 == SLang_pop_array_of_type (&at, SLANG_DOUBLE_TYPE))
 	  return;
 	break;
      }
-   
+
    if (NULL == (bt = SLang_create_array1 (at->data_type, 0, NULL, at->dims, at->num_dims, 1)))
      {
 	SLang_free_array (at);
@@ -1770,7 +1841,7 @@ static void frexp_intrin (void)
 	SLang_free_array (bt);
 	return;
      }
-   
+
    imax = at->num_elements;
    ep = (int *)et->data;
 
@@ -1792,7 +1863,7 @@ static void frexp_intrin (void)
 	     b[i] = FREXPF_FUN(a[i], ep+i);
 	  }
      }
-   
+
    (void) SLang_push_array (bt, 0);
    (void) SLang_push_array (et, 0);
    SLang_free_array (et);
@@ -1821,7 +1892,7 @@ static void ldexp_intrin (void)
 	  return;
 	e_ptr = (int *)e_at->data;
      }
-   else 
+   else
      {
 	if (-1 == SLang_pop_int (&e_data))
 	  return;
@@ -1899,7 +1970,7 @@ static void ldexp_intrin (void)
 	  }
 	goto push_free_and_return;
      }
-   
+
    if (e_at->num_elements != ast.num)
      {
 	SLang_verror (SL_TypeMismatch_Error, "ldexp: Array sizes do not match");
@@ -1974,7 +2045,7 @@ static SLang_Math_Unary_Type SLmath_Table [] =
    MAKE_MATH_UNARY("round", SLMATH_ROUND),
    MAKE_MATH_UNARY("expm1", SLMATH_EXPM1),
    MAKE_MATH_UNARY("log1p", SLMATH_LOG1P),
-   
+
 #if SLANG_HAS_COMPLEX
    MAKE_MATH_UNARY("Real", SLMATH_REAL),
    MAKE_MATH_UNARY("Imag", SLMATH_IMAG),
@@ -2007,7 +2078,7 @@ static SLang_Intrin_Fun_Type SLang_Math_Table [] =
    SLANG_END_INTRIN_FUN_TABLE
 };
 
-static SLang_IConstant_Type IConsts [] = 
+static SLang_IConstant_Type IConsts [] =
 {
    MAKE_ICONSTANT("FE_DIVBYZERO", SL_FE_DIVBYZERO),
    MAKE_ICONSTANT("FE_INVALID", SL_FE_INVALID),
@@ -2023,7 +2094,7 @@ static int add_nan_and_inf (void)
    if ((-1 == SLns_add_dconstant (NULL, "_NaN", _pSLang_NaN))
        || (-1 == SLns_add_dconstant (NULL, "_Inf", _pSLang_Inf)))
      return -1;
-   
+
    SLfpu_clear_except_bits ();
 
    return 0;
