@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2009 John E. Davis
+Copyright (C) 2004-2010 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -16,7 +16,7 @@ General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-USA.  
+USA.
 */
 
 #ifndef _GNU_SOURCE
@@ -83,12 +83,11 @@ void SLmake_lut (unsigned char *lut, unsigned char *range, unsigned char reverse
      }
 }
 
-
 char *_pSLskip_whitespace (SLCONST char *s)
 {
    while (isspace (*s))
      s++;
-   
+
    return (char *) s;
 }
 
@@ -214,7 +213,7 @@ char *_pSLexpand_escaped_char(char *p, char *pmax, SLwchar_Type *ch, int *isunic
 
    *ch = num;
    return p;
-   
+
 malformed_error:
    _pSLang_verror (SL_SYNTAX_ERROR, "Malformed escaped character.");
    return NULL;
@@ -240,7 +239,7 @@ int SLexpand_escaped_string (char *s, char *t, char *tmax, int utf8_encode)
 	     *s++ = ch;
 	     continue;
 	  }
-	
+
 	if (NULL == (t = _pSLexpand_escaped_char (t, tmax, &wch, &isunicode)))
 	  {
 	     *s = 0;
@@ -307,7 +306,7 @@ int SLvsnprintf (char *buf, unsigned int buflen, SLFUTURE_CONST char *fmt, va_li
    int status;
 
    status = vsnprintf(buf, buflen, fmt, ap);
-   if ((unsigned int)status >= buflen) 
+   if ((unsigned int)status >= buflen)
      status = -1;
    return status;
 #else
@@ -406,7 +405,6 @@ int SLang_add_cleanup_function (void (*f)(void))
    return 0;
 }
 
-
 int SLang_guess_type (SLFUTURE_CONST char *t)
 {
    SLCONST char *p;
@@ -418,6 +416,7 @@ int SLang_guess_type (SLFUTURE_CONST char *t)
 #define MODIFIER_LL	0x08
 #define MODIFIER_SIZE_MASK 0x0F
 #define MODIFIER_HEX	0x10
+#define MODIFIER_BINARY	0x20
 
    modifier = 0;
    if ((*t == '-') || (*t == '+')) t++;
@@ -429,13 +428,23 @@ int SLang_guess_type (SLFUTURE_CONST char *t)
 #endif
 	while ((*p >= '0') && (*p <= '9')) p++;
 	if (t == p) return (SLANG_STRING_TYPE);
-	if ((*p == 'x') && (p == t + 1))   /* 0x?? */
+	if (p == t + 1)
 	  {
-	     modifier |= MODIFIER_HEX;
-	     p++;
-	     while (ch = *p,
-		    ((ch >= '0') && (ch <= '9'))
-		    || (((ch | 0x20) >= 'a') && ((ch | 0x20) <= 'f'))) p++;
+	     if (*p == 'x')
+	       {
+		  modifier |= MODIFIER_HEX;
+		  p++;
+		  while (ch = *p,
+			 ((ch >= '0') && (ch <= '9'))
+			 || (((ch | 0x20) >= 'a') && ((ch | 0x20) <= 'f'))) p++;
+	       }
+	     else if (*p == 'b')
+	       {
+		  modifier |= MODIFIER_BINARY;
+		  p++;
+		  while ((*p == '0') || (*p == '1'))
+		    p++;
+	       }
 	  }
 
 	/* Now look for U, H, L, LL, UH, UL, ULL, modifiers */
@@ -492,7 +501,7 @@ int SLang_guess_type (SLFUTURE_CONST char *t)
 	   default:
 	     return SLANG_STRING_TYPE;
 	  }
-	     
+
 	if (modifier)
 	  return SLANG_STRING_TYPE;
 #if SLANG_HAS_FLOAT
@@ -566,7 +575,17 @@ static int hex_atoul (unsigned char *s, unsigned long *ul)
 		  return -1;
 	       }
 	  }
-	else 
+	else if ((*s | 0x20) == 'b')
+	  {
+	     base = 2;
+	     s++;
+	     if (*s == 0)
+	       {
+		  SLang_set_error (SL_SYNTAX_ERROR);
+		  return -1;
+	       }
+	  }
+	else
 	  {
 	     base = 8;
 	     num_processed++;
@@ -596,22 +615,27 @@ static int hex_atoul (unsigned char *s, unsigned long *ul)
 	     *ul = value;
 	     return 0;
 
-	   case '8':
 	   case '9':
-	     if (base == 8) 
+	   case '8':
+	     if (base <= 8)
 	       {
-		  _pSLang_verror (SL_SYNTAX_ERROR, "8 or 9 are not permitted in an octal number");
+		  _pSLang_verror (SL_SYNTAX_ERROR, "8 or 9 are not permitted in binary or octal numbers");
 		  return -1;
 	       }
 	     /* drop */
-	   case '0':
-	   case '1':
-	   case '2':
-	   case '3':
-	   case '4':
-	   case '5':
-	   case '6':
 	   case '7':
+	   case '6':
+	   case '5':
+	   case '4':
+	   case '3':
+	   case '2':
+	     if (base <= 2)
+	       {
+		  _pSLang_verror (SL_SYNTAX_ERROR, "Only digits 0 and 1 are permitted in binary numbers");
+		  return -1;
+	       }
+	   case '1':
+	   case '0':
 	     ch1 -= '0';
 	     break;
 
@@ -621,9 +645,9 @@ static int hex_atoul (unsigned char *s, unsigned long *ul)
 	   case 'd':
 	   case 'e':
 	   case 'f':
-	     if (base != 16) 
+	     if (base != 16)
 	       {
-		  _pSLang_verror (SL_SYNTAX_ERROR, "Only digits may appear in an octal or decimal number");
+		  _pSLang_verror (SL_SYNTAX_ERROR, "Only digits may appear in a binary, octal, or decimal number");
 		  return -1;
 	       }
 	     ch1 = (ch1 - 'a') + 10;
@@ -671,10 +695,19 @@ static int hex_atoull (unsigned char *s, unsigned long long *ul)
 		  return -1;
 	       }
 	  }
+	else if ((*s | 0x20) == 'b')
+	  {
+	     base = 2;
+	     s++;
+	     if (*s == 0)
+	       {
+		  SLang_set_error (SL_SYNTAX_ERROR);
+		  return -1;
+	       }
+	  }
 	else base = 8;
      }
 
-   
    value = 0;
    max_value_before_overflow = ULLONG_MAX/base;
    max_ch1_before_overflow = ULLONG_MAX - base*max_value_before_overflow;
@@ -694,22 +727,27 @@ static int hex_atoull (unsigned char *s, unsigned long long *ul)
 	     *ul = value;
 	     return 0;
 
-	   case '8':
 	   case '9':
-	     if (base == 8) 
+	   case '8':
+	     if (base <= 8)
 	       {
-		  _pSLang_verror (SL_SYNTAX_ERROR, "8 or 9 are not permitted in an octal number");
+		  _pSLang_verror (SL_SYNTAX_ERROR, "8 or 9 are not permitted in binary or octal numbers");
 		  return -1;
 	       }
 	     /* drop */
-	   case '0':
-	   case '1':
-	   case '2':
-	   case '3':
-	   case '4':
-	   case '5':
-	   case '6':
 	   case '7':
+	   case '6':
+	   case '5':
+	   case '4':
+	   case '3':
+	   case '2':
+	     if (base <= 2)
+	       {
+		  _pSLang_verror (SL_SYNTAX_ERROR, "Only digits 0 and 1 are permitted in binary numbers");
+		  return -1;
+	       }
+	   case '1':
+	   case '0':
 	     ch1 -= '0';
 	     break;
 
@@ -719,9 +757,9 @@ static int hex_atoull (unsigned char *s, unsigned long long *ul)
 	   case 'd':
 	   case 'e':
 	   case 'f':
-	     if (base != 16) 
+	     if (base != 16)
 	       {
-		  _pSLang_verror (SL_SYNTAX_ERROR, "Only digits may appear in an octal or decimal number");
+		  _pSLang_verror (SL_SYNTAX_ERROR, "Only digits may appear in a binary, octal, or decimal number");
 		  return -1;
 	       }
 	     ch1 = (ch1 - 'a') + 10;
@@ -742,7 +780,7 @@ static int hex_atoull (unsigned char *s, unsigned long long *ul)
    return 0;
 }
 #endif				       /* HAVE_LONG_LONG */
-   
+
 /* Note: These routines do not check integer overflow.  I would use the C
  * library functions atol and atoul but some implementations check overflow
  * and some do not.  The following implementations provide a consistent
@@ -752,7 +790,7 @@ static unsigned char *get_sign (SLCONST unsigned char *s, int *signp)
 {
    s = (unsigned char *) _pSLskip_whitespace ((char *)s);
 
-   if (*s == '-') 
+   if (*s == '-')
      {
 	*signp = -1;
 	s++;
