@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004, 2005, 2006, 2007, 2008 John E. Davis
+Copyright (C) 2004-2010 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -16,7 +16,7 @@ General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-USA.  
+USA.
 */
 
 #include "slinclud.h"
@@ -35,12 +35,10 @@ static void free_token (_pSLang_Token_Type *t)
 
    if (nrefs == 1)
      {
-	if (t->free_sval_flag)
+	if (t->free_val_func != NULL)
 	  {
-	     if (t->type == BSTRING_TOKEN)
-	       SLbstring_free (t->v.b_val);
-	     else
-	       _pSLfree_hashed_string ((char *) t->v.s_val, strlen (t->v.s_val), t->hash);
+	     (*t->free_val_func)(t);
+	     t->free_val_func = NULL;
 	     t->v.s_val = NULL;
 	  }
      }
@@ -65,6 +63,8 @@ static _pSLang_Token_Type Next_Token;
 static int Last_Line_Number = -1;
 #endif
 
+static int In_Looping_Context = 0;
+
 static int unget_token (_pSLang_Token_Type *ctok)
 {
    if (_pSLang_Error)
@@ -96,6 +96,137 @@ static int get_token (_pSLang_Token_Type *ctok)
    return _pSLget_token (ctok);
 }
 
+static int check_int_token_overflow (_pSLang_Token_Type *ctok, int sign)
+{
+   long ival, lval;
+   SLtype stype;
+
+   ctok->v.long_val = lval = sign * ctok->v.long_val;
+
+   if (ctok->flags & (SLTOKEN_IS_HEX|SLTOKEN_IS_BINARY))
+     return 0;
+
+   switch (ctok->type)
+     {
+      case CHAR_TOKEN:
+	stype = SLANG_CHAR_TYPE;
+	ival = (long)(char) lval;
+	break;
+      case SHORT_TOKEN:
+	stype = SLANG_SHORT_TYPE;
+	ival = (long)(short) lval;
+	break;
+      case INT_TOKEN:
+	stype = SLANG_INT_TYPE;
+	ival = (long)(int) lval;
+	break;
+      case LONG_TOKEN:
+	stype = LONG_TOKEN;
+        ival = lval;
+	break;
+
+      default:
+	return 0;
+     }
+
+   if (ival == lval)
+     {
+	if (((lval >= 0) && (sign > 0))
+	    || ((lval <= 0) && (sign < 0)))
+	  return 0;
+     }
+   SLang_verror (SL_SYNTAX_ERROR, "Literal integer constant is too large for %s", SLclass_get_datatype_name(stype));
+   return -1;
+}
+
+static int check_uint_token_overflow (_pSLang_Token_Type *ctok, int sign)
+{
+   unsigned long ival, lval;
+   SLtype stype;
+
+   ctok->v.long_val = sign * ctok->v.long_val;
+   lval = (unsigned long) ctok->v.long_val;
+
+   switch (ctok->type)
+     {
+      case UCHAR_TOKEN:
+	ival = (unsigned long)(unsigned char) lval;
+	stype = SLANG_UCHAR_TYPE;
+	break;
+      case USHORT_TOKEN:
+	stype = SLANG_USHORT_TYPE;
+	ival = (unsigned long)(unsigned short) lval;
+	break;
+      case UINT_TOKEN:
+	stype = SLANG_UINT_TYPE;
+	ival = (unsigned long)(unsigned int) lval;
+	break;
+      case ULONG_TOKEN:
+	stype = SLANG_ULONG_TYPE;
+	ival = lval;
+	break;
+      default:
+	return 0;
+     }
+
+   if (ival == lval)
+     return 0;
+
+   SLang_verror (SL_SYNTAX_ERROR, "Literal integer constant is too large for %s", SLclass_get_datatype_name(stype));
+   return -1;
+}
+
+#ifdef HAVE_LONG_LONG
+static int check_llong_token_overflow (_pSLang_Token_Type *ctok, int sign)
+{
+   long long lval = sign * ctok->v.llong_val;
+   ctok->v.llong_val = lval;
+
+   if ((ctok->flags & (SLTOKEN_IS_HEX|SLTOKEN_IS_BINARY))
+       || ((lval >= 0) && (sign > 0))
+       || ((lval <= 0) && (sign < 0)))
+     return 0;
+
+   SLang_verror (SL_SYNTAX_ERROR, "Literal integer constant is too large for %s", SLclass_get_datatype_name(SLANG_LLONG_TYPE));
+   return -1;
+}
+
+static int check_ullong_token_overflow (_pSLang_Token_Type *ctok, int sign)
+{
+   ctok->v.ullong_val *= sign;
+   return 0;
+}
+#endif
+
+static int check_number_token_overflow (_pSLang_Token_Type *tok, int sign)
+{
+   tok->flags |= SLTOKEN_OVERFLOW_CHECKED;
+
+   switch (tok->type)
+     {
+      case CHAR_TOKEN:
+      case SHORT_TOKEN:
+      case INT_TOKEN:
+      case LONG_TOKEN:
+	return check_int_token_overflow (tok, sign);
+
+      case UCHAR_TOKEN:
+      case USHORT_TOKEN:
+      case UINT_TOKEN:
+      case ULONG_TOKEN:
+	return check_uint_token_overflow (tok, sign);
+
+#ifdef HAVE_LONG_LONG
+      case LLONG_TOKEN:
+	return check_llong_token_overflow (tok, sign);
+
+      case ULLONG_TOKEN:
+	return check_ullong_token_overflow (tok, sign);
+#endif
+     }
+   return 0;
+}
+
 static int compile_token (_pSLang_Token_Type *t)
 {
 #if SLANG_HAS_DEBUG_CODE
@@ -108,6 +239,11 @@ static int compile_token (_pSLang_Token_Type *t)
 	(*_pSLcompile_ptr) (&tok);
      }
 #endif
+   if ((t->flags & (SLTOKEN_TYPE_INTEGER|SLTOKEN_OVERFLOW_CHECKED)) == SLTOKEN_TYPE_INTEGER)
+     {
+	if (-1 == check_number_token_overflow (t, 1))
+	  return -1;
+     }
    (*_pSLcompile_ptr) (t);
    return 0;
 }
@@ -172,7 +308,7 @@ static Token_List_Type *push_token_list (void)
 {
    if (Token_List_Stack_Depth == MAX_TOKEN_LISTS)
      {
-	_pSLparse_error (SL_BUILTIN_LIMIT_EXCEEDED, 
+	_pSLparse_error (SL_BUILTIN_LIMIT_EXCEEDED,
 			"Token list stack size exceeded", NULL, 0);
 	return NULL;
      }
@@ -258,27 +394,51 @@ static int append_copy_of_string_token (_pSLang_Token_Type *t)
 
    t1 = Token_List->stack + Token_List->len;
    *t1 = *t;
-   
-   if (NULL == (t1->v.s_val = SLang_create_slstring (t->v.s_val)))
+
+   if (t->v.s_val == NULL)
      return -1;
-   
-   t1->free_sval_flag = 1;
+
+   if (EOF_TOKEN == _pSLtoken_init_slstring_token (t1, t->type, t->v.s_val, strlen (t->v.s_val)))
+     return -1;
+
    t1->num_refs = 1;
 
    Token_List->len += 1;
    return 0;
 }
 
-static int append_int_token (int n)
-{     
+static int append_int_as_token (int n)
+{
    _pSLang_Token_Type num_tok;
 
    init_token (&num_tok);
    num_tok.type = INT_TOKEN;
+   num_tok.flags |= SLTOKEN_TYPE_INTEGER|SLTOKEN_OVERFLOW_CHECKED;
    num_tok.v.long_val = n;
    return append_token (&num_tok);
 }
 
+#if 0
+static int append_string_as_token (char *str)
+{
+   _pSLang_Token_Type *t;
+
+   if (-1 == check_token_list_space (Token_List, 1))
+     return -1;
+
+   t = Token_List->stack + Token_List->len;
+   init_token (t);
+
+   if (EOF_TOKEN == _pSLtoken_init_slstring_token (t, STRING_TOKEN, str, strlen (str)))
+     return -1;
+
+   t->num_refs = 1;
+
+   Token_List->len += 1;
+   return 0;
+}
+#endif
+
 static int append_token_of_type (unsigned char t)
 {
    _pSLang_Token_Type *tok;
@@ -289,6 +449,7 @@ static int append_token_of_type (unsigned char t)
    /* The memset when the list was created ensures that the other fields
     * are properly initialized.
     */
+#if 0
    if ((t == CHS_TOKEN) && Token_List->len)
      {
 	tok = Token_List->stack + (Token_List->len-1);
@@ -305,6 +466,7 @@ static int append_token_of_type (unsigned char t)
 	  }
 #endif
      }
+#endif
 
    tok = Token_List->stack + Token_List->len;
    init_token (tok);
@@ -375,6 +537,8 @@ static int compile_token_list (void)
  * with the elements at pos1, e.g.,
  * ...ABCDEabc ==> ...abcABCDE
  * where pos1 denotes A and pos2 denotes a.
+ *
+ * NOTE: The caller must make special provisions for NO_OP_LITERAL tokens.
  */
 static int token_list_element_exchange (unsigned int pos1, unsigned int pos2)
 {
@@ -391,6 +555,12 @@ static int token_list_element_exchange (unsigned int pos1, unsigned int pos2)
        || (pos2 >= len))
      return -1;
 
+   if (pos1 > pos2)
+     {
+	SLang_verror (SL_INTERNAL_ERROR, "pos1<pos2 in token_list_element_exchange");
+	return -1;
+     }
+
    /* This may not be the most efficient algorithm but the number to swap
     * is most-likely going to be small, e.g, 3
     * The algorithm is to rotate the list.  The particular rotation
@@ -442,6 +612,7 @@ static void compile_token_of_type (unsigned char t)
 #if SLANG_HAS_DEBUG_CODE
    tok.line_number = -1;
 #endif
+   tok.flags = 0;
    tok.type = t;
    compile_token(&tok);
 }
@@ -460,7 +631,7 @@ static void compile_eos (void)
 static int compile_bos (_pSLang_Token_Type *t, int level)
 {
    _pSLang_Token_Type tok;
-   
+
    if (level > _pSLang_Compile_BOSEOS)
      return 0;
 
@@ -482,14 +653,14 @@ static void append_eos (void)
 static int append_bos (_pSLang_Token_Type *t, int level)
 {
    _pSLang_Token_Type tok;
-   
+
    if (level > _pSLang_Compile_BOSEOS)
      return 0;
 
    init_token (&tok);
    tok.type = BOS_TOKEN;
    tok.v.long_val = t->line_number;
-   
+
    append_token (&tok);
    free_token (&tok);
    return 1;
@@ -497,6 +668,7 @@ static int append_bos (_pSLang_Token_Type *t, int level)
 #endif
 
 static void statement (_pSLang_Token_Type *);
+static void loop_statement (_pSLang_Token_Type *);
 static void compound_statement (_pSLang_Token_Type *);
 static void expression_with_parenthesis (_pSLang_Token_Type *);
 static void handle_semicolon (_pSLang_Token_Type *);
@@ -507,7 +679,7 @@ static void variable_list (_pSLang_Token_Type *, unsigned char);
 static void struct_declaration (_pSLang_Token_Type *, int);
 static void define_function_args (_pSLang_Token_Type *);
 static void typedef_definition (_pSLang_Token_Type *);
-static void function_args_expression (_pSLang_Token_Type *, int, int);
+static void function_args_expression (_pSLang_Token_Type *, int, int, int, unsigned int *);
 static void expression (_pSLang_Token_Type *);
 static void expression_with_commas (_pSLang_Token_Type *, int);
 static void simple_expression (_pSLang_Token_Type *);
@@ -516,6 +688,7 @@ static void postfix_expression (_pSLang_Token_Type *);
 static int check_for_lvalue (unsigned char, _pSLang_Token_Type *);
 /* static void primary_expression (_pSLang_Token_Type *); */
 static void block (_pSLang_Token_Type *);
+static void loop_block (_pSLang_Token_Type *);
 static void inline_array_expression (_pSLang_Token_Type *);
 static void inline_list_expression (_pSLang_Token_Type *);
 static void array_index_expression (_pSLang_Token_Type *);
@@ -559,17 +732,17 @@ static int get_identifier_token (_pSLang_Token_Type *tok)
 static void define_function (_pSLang_Token_Type *ctok, unsigned char type)
 {
    _pSLang_Token_Type fname;
-   
+
    switch (type)
      {
       case STATIC_TOKEN:
 	type = DEFINE_STATIC_TOKEN;
 	break;
-	
+
       case PUBLIC_TOKEN:
 	type = DEFINE_PUBLIC_TOKEN;
 	break;
-	
+
       case PRIVATE_TOKEN:
 	type = DEFINE_PRIVATE_TOKEN;
      }
@@ -587,7 +760,12 @@ static void define_function (_pSLang_Token_Type *ctok, unsigned char type)
    compile_token_of_type(FARG_TOKEN);
 
    if (ctok->type == OBRACE_TOKEN)
-     compound_statement(ctok);
+     {
+	int loop_context = In_Looping_Context;
+	In_Looping_Context = 0;
+	compound_statement(ctok);
+	In_Looping_Context = loop_context;
+     }
 
    else if (ctok->type != SEMICOLON_TOKEN)
      {
@@ -604,15 +782,15 @@ static void define_function (_pSLang_Token_Type *ctok, unsigned char type)
 /* This is called from "statement", which is expected to return no
  * new token.
  */
-static void check_for_loop_then_else (_pSLang_Token_Type *ctok)
+static int check_for_loop_then_else (_pSLang_Token_Type *ctok)
 {
    int b = 0;
-   
+
    get_token (ctok);
 
    while (1)
      {
-	unsigned char type = ctok->type;
+	_pSLtok_Type type = ctok->type;
 #ifdef LOOP_ELSE_TOKEN
 	if ((type == ELSE_TOKEN) && ((b & 1) == 0))
 	  {
@@ -629,13 +807,18 @@ static void check_for_loop_then_else (_pSLang_Token_Type *ctok)
 	     get_token (ctok);
 	     block (ctok);
 	     compile_token_of_type (LOOP_THEN_TOKEN);
-	     get_token (ctok);
 	     b |= 2;
+#ifdef LOOP_ELSE_TOKEN
+	     get_token (ctok);
 	     continue;
+#else
+	     return b;
+#endif
 	  }
 	break;
      }
    unget_token (ctok);
+   return b;
 }
 
 /* statement:
@@ -725,7 +908,8 @@ static void statement (_pSLang_Token_Type *ctok)
       case _FOR_TOKEN:
 	get_token (ctok);
 	handle_for_statement (ctok);
-	check_for_loop_then_else (ctok);
+	if (0 == check_for_loop_then_else (ctok))
+	  compile_token_of_type (NOP_TOKEN);
 	break;
 
       case LOOP_TOKEN:
@@ -737,15 +921,17 @@ static void statement (_pSLang_Token_Type *ctok)
 #if SLANG_HAS_BOSEOS
 	if (eos) compile_eos ();
 #endif
-	block (ctok);
+	loop_block (ctok);
 	compile_token_of_type (LOOP_TOKEN);
-	check_for_loop_then_else (ctok);
+	if (0 == check_for_loop_then_else (ctok))
+	  compile_token_of_type (NOP_TOKEN);
 	break;
 
       case FOREACH_TOKEN:
 	get_token (ctok);
 	handle_foreach_statement (ctok);
-	check_for_loop_then_else (ctok);
+	if (0 == check_for_loop_then_else (ctok))
+	  compile_token_of_type (NOP_TOKEN);
 	break;
 
       case WHILE_TOKEN:
@@ -759,14 +945,15 @@ static void statement (_pSLang_Token_Type *ctok)
 	if (eos) compile_eos ();
 #endif
 	compile_token_of_type (CBRACE_TOKEN);
-	block (ctok);
+	loop_block (ctok);
 	compile_token_of_type (WHILE_TOKEN);
-	check_for_loop_then_else (ctok);
+	if (0 == check_for_loop_then_else (ctok))
+	  compile_token_of_type (NOP_TOKEN);
 	break;
 
       case DO_TOKEN:
 	get_token (ctok);
-	block (ctok);
+	loop_block (ctok);
 
 	if (WHILE_TOKEN != get_token (ctok))
 	  {
@@ -787,11 +974,11 @@ static void statement (_pSLang_Token_Type *ctok)
 	compile_token_of_type (CBRACE_TOKEN);
 	compile_token_of_type (DOWHILE_TOKEN);
 	handle_semicolon (ctok);
-	check_for_loop_then_else (ctok);
+	if (0 == check_for_loop_then_else (ctok))
+	  compile_token_of_type (NOP_TOKEN);
 	break;
 
       case FOR_TOKEN:
-
 	/* Look for (exp_opt ; exp_opt ; exp_opt ) */
 
 	if (OPAREN_TOKEN != get_token (ctok))
@@ -860,13 +1047,20 @@ static void statement (_pSLang_Token_Type *ctok)
 	compile_token_list ();
 
 	get_token (ctok);
-	block (ctok);
+	loop_block (ctok);
 	compile_token_of_type (FOR_TOKEN);
-	check_for_loop_then_else (ctok);
+	if (0 == check_for_loop_then_else (ctok))
+	  compile_token_of_type (NOP_TOKEN);
 	break;
 
       case FOREVER_TOKEN:
-	/* drop */
+	get_token (ctok);
+	loop_block (ctok);
+	compile_token_of_type (FOREVER_TOKEN);
+	if (0 == check_for_loop_then_else (ctok))
+	  compile_token_of_type (NOP_TOKEN);
+	break;
+
       case ERRBLK_TOKEN:
       case EXITBLK_TOKEN:
       case USRBLK0_TOKEN:
@@ -878,19 +1072,37 @@ static void statement (_pSLang_Token_Type *ctok)
 	get_token (ctok);
 	block (ctok);
 	compile_token_of_type (type);
-	if (type == FOREVER_TOKEN)
-	  check_for_loop_then_else (ctok);
 	break;
 
       case BREAK_TOKEN:
       case CONT_TOKEN:
+	if (In_Looping_Context == 0)
+	  {
+	     _pSLparse_error (SL_SYNTAX_ERROR, "`break' or `continue' requires a looping context", ctok, 0);
+	     break;
+	  }
+
+	type = ctok->type;
 #if SLANG_HAS_BOSEOS
 	eos = compile_bos (ctok, 3);
+#endif
+	if (SEMICOLON_TOKEN != get_token (ctok))
+	  {
+	     if ((ctok->type != INT_TOKEN)
+		 || (ctok->v.long_val <= 0))
+	       {
+		  _pSLparse_error (SL_SYNTAX_ERROR, "Expecting a positive non-zero integer or a semi-colon.", ctok, 0);
+		  break;
+	       }
+	     ctok->type = (type == BREAK_TOKEN) ? BREAK_N_TOKEN : CONT_N_TOKEN;
+	     compile_token (ctok);
+	     get_token (ctok);
+	     handle_semicolon (ctok);
+	  }
+	else compile_token_of_type (type);
+#if SLANG_HAS_BOSEOS
 	if (eos) compile_eos ();
 #endif
-	compile_token_of_type (ctok->type);
-	get_token (ctok);
-	handle_semicolon (ctok);
 	break;
 
       case RETURN_TOKEN:
@@ -957,7 +1169,7 @@ static void statement (_pSLang_Token_Type *ctok)
       case DEFINE_TOKEN:
 	define_function (ctok, DEFINE_TOKEN);
 	break;
-	
+
       case TRY_TOKEN:
 	get_token (ctok);
 	handle_try_statement (ctok);
@@ -1052,6 +1264,13 @@ static void statement (_pSLang_Token_Type *ctok)
    LLT->parse_level -= 1;
 }
 
+static void loop_statement (_pSLang_Token_Type *ctok)
+{
+   In_Looping_Context++;
+   statement (ctok);
+   In_Looping_Context--;
+}
+
 /* This function does not return a new token.  Calling routine must do that */
 static void block (_pSLang_Token_Type *ctok)
 {
@@ -1060,6 +1279,13 @@ static void block (_pSLang_Token_Type *ctok)
    compile_token_of_type (CBRACE_TOKEN);
 }
 
+static void loop_block (_pSLang_Token_Type *ctok)
+{
+   compile_token_of_type (OBRACE_TOKEN);
+   loop_statement (ctok);
+   compile_token_of_type (CBRACE_TOKEN);
+}
+
 static void handle_for_statement (_pSLang_Token_Type *ctok)
 {
    _pSLang_Token_Type tok_buf;
@@ -1089,37 +1315,24 @@ static void handle_for_statement (_pSLang_Token_Type *ctok)
 	compile_token (ident_token);
 	free_token (ident_token);
      }
-   statement (ctok);
+   loop_statement (ctok);
    compile_token_of_type (CBRACE_TOKEN);
 
    compile_token_of_type (_FOR_TOKEN);
 }
 
-static int init_identifier_token (_pSLang_Token_Type *t, SLFUTURE_CONST char *name, int alloc)
+static int init_identifier_token (_pSLang_Token_Type *t, SLFUTURE_CONST char *name)
 {
-   unsigned int len;
-   
-   len = strlen (name);
-
    init_token (t);
-   if (alloc)
-     {
-	if (NULL == (t->v.s_val = _pSLstring_make_hashed_string (name, len, &t->hash)))
-	  return -1;
-   
-	t->free_sval_flag = 1;
-	return 0;
-     }
-   
-   t->type = IDENT_TOKEN;
-   t->v.s_val = name;
-   t->hash = _pSLstring_hash ((unsigned char *)name, (unsigned char *)name + len);
-   t->free_sval_flag = 0;
+
+   if (EOF_TOKEN == _pSLtoken_init_slstring_token (t, IDENT_TOKEN, name, strlen(name)))
+     return -1;
+
    return 0;
 }
 
 /* The try-statement looks like:
- * 
+ *
  *    TRY ev_optional catch_blocks_opt finally_block_opt
  *
  *    ev_optional:
@@ -1134,13 +1347,13 @@ static int init_identifier_token (_pSLang_Token_Type *t, SLFUTURE_CONST char *na
  *    exception-list:
  *       expression
  *       exception-list , exception
- * 
+ *
  *    finally_block_opt:
  *       FINALLY[:] block
- * 
+ *
  * The above gets compiled into the form:
- * 
- *    {ev_block} {try-statements} 
+ *
+ *    {ev_block} {try-statements}
  *       {exception-list}{catch-block}
  *               .
  *               .
@@ -1160,9 +1373,9 @@ static void handle_try_statement (_pSLang_Token_Type *ctok)
    if (ctok->type == OPAREN_TOKEN)
      {
 	_pSLang_Token_Type e;
-	if (-1 == init_identifier_token (&e, "__get_exception_info", 0))
+	if (-1 == init_identifier_token (&e, "__get_exception_info"))
 	  return;
-	
+
 	append_token (&e);
 	free_token (&e);
 
@@ -1190,7 +1403,7 @@ static void handle_try_statement (_pSLang_Token_Type *ctok)
 	/* Expecting catch expression-list: */
 	compile_token_of_type (OBRACE_TOKEN);
 	get_token (ctok);
-	
+
 	push_token_list ();
 
 	while (_pSLang_Error == 0)
@@ -1203,7 +1416,7 @@ static void handle_try_statement (_pSLang_Token_Type *ctok)
 	       break;
 	     get_token (ctok);
 	  }
-	
+
 	if (ctok->type == COLON_TOKEN)
 	  get_token (ctok);
 	else if (ctok->type != SEMICOLON_TOKEN)
@@ -1214,19 +1427,19 @@ static void handle_try_statement (_pSLang_Token_Type *ctok)
 	compile_token_list ();
 
 	compile_token_of_type (CBRACE_TOKEN);
-	
+
 	/* catch block */
 	block (ctok);
 	num_catches++;
      }
-   
+
    if ((num_catches == 0)
        && (ctok->type != FINALLY_TOKEN))
      {
 	_pSLparse_error (SL_SYNTAX_ERROR, "Expecting \"catch\" or \"finally\"", ctok, 0);
 	return;
      }
-   
+
    /* finally */
    if (ctok->type == FINALLY_TOKEN)
      {
@@ -1240,7 +1453,7 @@ static void handle_try_statement (_pSLang_Token_Type *ctok)
 	/* since this is called directly from statement, we need to get only
 	 * tokens that were used.  So, since we are not using this, put it back.
 	 */
-	unget_token (ctok);	       
+	unget_token (ctok);
 	compile_token_of_type (OBRACE_TOKEN);
 	compile_token_of_type (CBRACE_TOKEN);
      }
@@ -1299,7 +1512,7 @@ static _pSLang_Token_Type *allocate_token (void)
    v = (_pSLang_Token_Type *)SLmalloc (sizeof (_pSLang_Token_Type));
    if (v == NULL)
      return NULL;
-   
+
    init_token (v);
    return v;
 }
@@ -1311,7 +1524,7 @@ static void handle_foreach_statement (_pSLang_Token_Type *ctok)
 #if SLANG_HAS_BOSEOS
    int eos;
 #endif
-   
+
 #if SLANG_HAS_BOSEOS
    eos = compile_bos (ctok, 2);
 #endif
@@ -1349,14 +1562,28 @@ static void handle_foreach_statement (_pSLang_Token_Type *ctok)
 	     goto free_return;
 	  }
 	get_token (ctok);
-	function_args_expression (ctok, 0, 0);
+	function_args_expression (ctok, 0, 0, 0, NULL);
      }
-   append_token_of_type (EARG_TOKEN);
-   
+   /* append_token_of_type (EARG_TOKEN); // This is now handled by the
+    * FOREACH_TOKEN itself.  Doing it presents a problem if a hook gets
+    * called when the loop_statements are being parsed.  This can and will
+    * happen when slsh calls its massage_input hook.  The would result in
+    * bytecode such as:
+    *  __args <using-expression> __eargs __args <hookargs> __eargs hook
+    * As a result, the first __args/__eargs info will be lost.  By allowing
+    * __foreach__ to handle it, we effectively get the proper nested
+    * __args/__eargs form:
+    *  __args <using-expression> __args <hookargs> __eargs hook __eargs __foreach
+    *
+    * Yes, this is subtle.  Always follow the rule:  Avoid __args/__eargs
+    * outside a token list and make sure the code that uses it is either also
+    * in the same token list, or implicitely calls __eargs.
+    */
+
    compile_token_list ();
 
    compile_token_of_type (OBRACE_TOKEN);
-   
+
    v = var_tokens;
    while (v != NULL)
      {
@@ -1364,10 +1591,12 @@ static void handle_foreach_statement (_pSLang_Token_Type *ctok)
 	compile_token (v);
 	v = v->next;
      }
-   statement (ctok);
+
+   loop_statement (ctok);
+
    compile_token_of_type (CBRACE_TOKEN);
-   compile_token_of_type (FOREACH_TOKEN);
-   
+   compile_token_of_type (FOREACH_EARGS_TOKEN);
+
    free_return:
    while (var_tokens != NULL)
      {
@@ -1448,6 +1677,7 @@ void _pSLparse_start (SLang_Load_Type *llt)
    unsigned int save_use_next_token;
    _pSLang_Token_Type save_next_token;
    Token_List_Type *save_list;
+   int save_looping_context = In_Looping_Context;
 #if SLANG_HAS_DEBUG_CODE
    int save_last_line_number = Last_Line_Number;
 
@@ -1461,16 +1691,20 @@ void _pSLparse_start (SLang_Load_Type *llt)
 
    init_token (&Next_Token);
    Use_Next_Token = 0;
+   In_Looping_Context = 0;
    init_token (&ctok);
    get_token (&ctok);
 
    llt->parse_level = 0;
    statement_list (&ctok);
 
-   if ((_pSLang_Error == 0)
-       && (ctok.type != EOF_TOKEN))
-     _pSLparse_error (SL_SYNTAX_ERROR, "Parse ended prematurely", &ctok, 0);
-   
+   if (_pSLang_Error == 0)
+     {
+	if (ctok.type != EOF_TOKEN)
+	  _pSLparse_error (SL_SYNTAX_ERROR, "Parse ended prematurely", &ctok, 0);
+	else
+	  compile_token_of_type (EOF_TOKEN);
+     }
 
    if (_pSLang_Error)
      {
@@ -1490,6 +1724,8 @@ void _pSLparse_start (SLang_Load_Type *llt)
      free_token (&Next_Token);
    Use_Next_Token = save_use_next_token;
    Next_Token = save_next_token;
+   In_Looping_Context = save_looping_context;
+
 #if SLANG_HAS_DEBUG_CODE
    Last_Line_Number = save_last_line_number;
 #endif
@@ -1575,6 +1811,28 @@ static void free_token_linked_list (_pSLang_Token_Type *tok)
      }
 }
 
+/* This works with any string-like token */
+static int prefix_token_sval_field (_pSLang_Token_Type *tok, char *prefix)
+{
+   char buf[2*SL_MAX_TOKEN_LEN];
+   unsigned int len, prefix_len;
+
+   prefix_len = strlen (prefix);
+   len = _pSLstring_bytelen (tok->v.s_val);   /* sign */
+   if (len + prefix_len >= sizeof(buf))
+     {
+	_pSLparse_error (SL_BUILTIN_LIMIT_EXCEEDED, "Number too long for buffer", tok, 1);
+	return -1;
+     }
+   memcpy (buf, prefix, prefix_len);
+   memcpy (buf+prefix_len, tok->v.s_val, len);  /* copys \0 */
+   (*tok->free_val_func)(tok);
+   if (EOF_TOKEN == _pSLtoken_init_slstring_token (tok, tok->type, buf, len+prefix_len))
+     return -1;
+
+   return 0;
+}
+
 /*
  * This function parses a structure definition block.  It returns the names
  * of the structure fields in the form of a linked list of tokens.
@@ -1592,12 +1850,32 @@ static _pSLang_Token_Type *
    _pSLang_Token_Type *name_list_root = NULL;
    _pSLang_Token_Type *name_list_tail = NULL;
    unsigned int n, m;
+   char buf[64];
 
    n = m = 0;
-   while ((_pSLang_Error == 0) 
-	  && (IDENT_TOKEN == ctok->type))
+   while (_pSLang_Error == 0)
      {
-	_pSLang_Token_Type *new_tok = allocate_token ();
+	_pSLang_Token_Type *new_tok;
+	int is_deref = 0;
+
+	if (assign_ok && (ctok->type == DEREF_TOKEN))
+	  {
+	     /* struct { @ expr, ... } */
+
+	     (void) SLsnprintf (buf, sizeof(buf), "@%d", n);
+	     free_token (ctok);
+	     if (EOF_TOKEN == _pSLtoken_init_slstring_token (ctok, STRING_TOKEN, buf, strlen (buf)))
+	       break;
+
+	     is_deref = 1;
+	  }
+	else
+	  {
+	     if (IDENT_TOKEN != ctok->type)
+	       break;
+	  }
+
+	new_tok = allocate_token ();
 	if (new_tok == NULL)
 	  break;
 
@@ -1614,34 +1892,42 @@ static _pSLang_Token_Type *
 
 	n++;
 
-	if (COMMA_TOKEN == get_token (ctok))
+	if ((COMMA_TOKEN == get_token (ctok))
+	    && (is_deref == 0))
 	  {
 	     get_token (ctok);
 	     continue;
 	  }
 
-	if (assign_ok && (ASSIGN_TOKEN == ctok->type))
+	if (assign_ok == 0)
+	  break;
+
+	if ((ASSIGN_TOKEN == ctok->type) || is_deref)
 	  {
 	     /* name = ... */
 #if SLANG_HAS_BOSEOS
-	     int eos = compile_bos (ctok, 1);
+	     int eos = append_bos (ctok, 1);
 #endif
-	     get_token (ctok);
+	     if (is_deref == 0)
+	       get_token (ctok);
+
 	     simple_expression (ctok);
 #if SLANG_HAS_BOSEOS
-	     if (eos) compile_eos ();
+	     if (eos) append_eos ();
 #endif
+
 	     if (-1 == append_copy_of_string_token (new_tok))
 	       break;
-	     
+
 	     m++;
 
-	     if (ctok->type == COMMA_TOKEN)
-	       {
-		  get_token (ctok);
-		  continue;
-	       }
-	  }	
+	     if (ctok->type != COMMA_TOKEN)
+	       break;
+
+	     get_token (ctok);
+	     continue;
+	  }
+
 	break;
      }
 
@@ -1650,10 +1936,10 @@ static _pSLang_Token_Type *
 	free_token_linked_list (name_list_root);
 	return NULL;
      }
-   
+
    if (n == 0)
      {
-	_pSLparse_error (SL_SYNTAX_ERROR, "Expecting an identifier", ctok, 0);
+	_pSLparse_error (SL_SYNTAX_ERROR, "Expecting a qualifier", ctok, 0);
 	return NULL;
      }
 
@@ -1683,15 +1969,15 @@ static int handle_struct_fields (_pSLang_Token_Type *ctok, int assign_ok)
    if (_pSLang_Error)
      return -1;
 
-   append_int_token (n);
+   append_int_as_token (n);
    if (m == 0)
      append_token_of_type (STRUCT_TOKEN);
    else
      {
-	append_int_token (m);
+	append_int_as_token (m);
 	append_token_of_type (STRUCT_WITH_ASSIGN_TOKEN);
      }
-   
+
    if (_pSLang_Error)
      return -1;
 
@@ -1705,10 +1991,10 @@ static int handle_struct_fields (_pSLang_Token_Type *ctok, int assign_ok)
  * 	struct-field-name [= simple_expr], struct-field-list
  * 	struct-field-name [= simple_expr]
  *
- * Generates code: 
+ * Generates code:
  *    "field-name-1" ... "field-name-N" N STRUCT_TOKEN
  * - OR -
- *    expr-k1 "field-name-k1" ... expr-kM "field-name-kM"  
+ *    expr-k1 "field-name-k1" ... expr-kM "field-name-kM"
  *        "name-1" ... "field-name-N" N M STRUCT_DEF_ASSIGN_TOKEN
  */
 static void struct_declaration (_pSLang_Token_Type *ctok, int assign_ok)
@@ -1722,7 +2008,7 @@ static void struct_declaration (_pSLang_Token_Type *ctok, int assign_ok)
 
    if (-1 == handle_struct_fields (ctok, assign_ok))
      return;
-   
+
    if (ctok->type != CBRACE_TOKEN)
      {
 	_pSLparse_error (SL_SYNTAX_ERROR, "Expecting }", ctok, 0);
@@ -1886,7 +2172,16 @@ static void assignment_expression (_pSLang_Token_Type *ctok)
 	return;
      }
    start_pos = Token_List->len;
-   
+
+   if (ctok->type == NO_OP_LITERAL)
+     {
+	/* This is called from try_multiple_assignment with a new token list.
+	 * The tokens added to that list collectively make up a object that
+	 * is treated as a literal.  Reset the list start position to
+	 * those start of those elements.
+	 */
+	start_pos = 0;
+     }
    simple_expression (ctok);
    switch (ctok->type)
      {
@@ -1977,7 +2272,7 @@ static void handle_binary_sequence (_pSLang_Token_Type *, unsigned char);
 static void handle_sc_sequence (_pSLang_Token_Type *ctok, unsigned char level)
 {
    unsigned char type = ctok->type;
-   
+
    while ((ctok->type == type) && (_pSLang_Error == 0))
      {
 	append_token_of_type (OBRACE_TOKEN);
@@ -2025,7 +2320,6 @@ static void handle_compare_sequence (_pSLang_Token_Type *ctok, unsigned char lev
    append_token_of_type (_COMPARE_TOKEN);
 }
 
-
 static void handle_binary_sequence (_pSLang_Token_Type *ctok, unsigned char max_level)
 {
    unsigned char op_stack [64];
@@ -2082,6 +2376,7 @@ static void handle_binary_sequence (_pSLang_Token_Type *ctok, unsigned char max_
 /* % Note: simple-expression groups operators OP1 at same level.  The
  * % actual implementation will not do this.
  * simple-expression:
+ *       simple-expression ? simple-expression : simple-expression
  *	 unary-expression
  *	 binary-expression BINARY-OP unary-expression
  *       andelse xxelse-expression-list
@@ -2140,6 +2435,29 @@ static void simple_expression (_pSLang_Token_Type *ctok)
      return;
 
    handle_binary_sequence (ctok, 0xFF);
+
+   if (ctok->type == QUESTION_TOKEN)
+     {
+	append_token_of_type (OBRACE_TOKEN);
+	get_token (ctok);
+	simple_expression (ctok);
+	if (ctok->type != COLON_TOKEN)
+	  {
+	     _pSLparse_error (SL_SYNTAX_ERROR, "Expecting a colon in the ternary expression", ctok, 0);
+	     return;
+	  }
+	append_token_of_type (CBRACE_TOKEN);
+	get_token (ctok);
+	append_token_of_type (OBRACE_TOKEN);
+	simple_expression (ctok);
+	append_token_of_type (CBRACE_TOKEN);
+	append_token_of_type (ELSE_TOKEN);
+     }
+}
+
+static int negate_float_type_token (_pSLang_Token_Type *tok)
+{
+   return prefix_token_sval_field (tok, "-");
 }
 
 /* unary-expression:
@@ -2179,20 +2497,29 @@ static void unary_expression (_pSLang_Token_Type *ctok)
 
 	   case SUB_TOKEN:
 	     (void) get_token (ctok);
-#if 0
-	     if (IS_INTEGER_TOKEN (ctok->type))
+	     if (ctok->flags & SLTOKEN_TYPE_NUMBER)
 	       {
-		  ctok->v.long_val = -ctok->v.long_val;
-		  break;
+		  _pSLang_Token_Type *last_token;
+		  postfix_expression (ctok);
+		  if ((NULL != (last_token = get_last_token ()))
+		      && (last_token->flags & SLTOKEN_TYPE_NUMBER))
+		    {
+		       if (last_token->flags & SLTOKEN_TYPE_FLOAT)
+			 {
+			    if (-1 == negate_float_type_token (last_token))
+			      return;
+			 }
+		       else if (-1 == check_number_token_overflow (last_token, -1))
+			 return;
+		    }
+		  else
+		    {
+		       if (num_unary_ops == 16)
+			 goto stack_overflow_error;
+		       save_unary_ops [num_unary_ops++] = CHS_TOKEN;
+		    }
+		  goto out_of_switch;
 	       }
-#ifdef HAVE_LONG_LONG
-	     else if ((ctok->type == LLONG_TOKEN) || (ctok->type == ULLONG_TOKEN))
-	       {
-		  ctok->v.llong_val = -ctok->v.llong_val;
-		  break;
-	       }
-#endif
-#endif
 	     if (num_unary_ops == 16)
 	       goto stack_overflow_error;
 	     save_unary_ops [num_unary_ops++] = CHS_TOKEN;
@@ -2341,7 +2668,7 @@ static int get_identifier_expr_token (_pSLang_Token_Type *ctok)
  *	 postfix-expression ( function-args-expression )
  *	 postfix-expression . identifier
  *       postfix-expression ^ unary-expression
- * 
+ *
  * Not yet supported:
  *	 postfix-expression ++
  *	 postfix-expression --
@@ -2365,7 +2692,7 @@ static int get_identifier_expr_token (_pSLang_Token_Type *ctok)
  */
 static void postfix_expression (_pSLang_Token_Type *ctok)
 {
-   unsigned int start_pos, end_pos;
+   unsigned int start_pos;
    unsigned char type;
    _pSLang_Token_Type *last_token;
 
@@ -2382,7 +2709,7 @@ static void postfix_expression (_pSLang_Token_Type *ctok)
 	postfix_expression (ctok);
 	append_token_of_type (DEREF_TOKEN);
 	break;
-#endif	
+#endif
       case IDENT_TOKEN:
 	append_identifier_token (ctok);
 	break;
@@ -2395,6 +2722,10 @@ static void postfix_expression (_pSLang_Token_Type *ctok)
       case USHORT_TOKEN:
       case UINT_TOKEN:
       case ULONG_TOKEN:
+#ifdef HAVE_LONG_LONG
+      case LLONG_TOKEN:
+      case ULLONG_TOKEN:
+#endif
       case STRING_TOKEN:
       case BSTRING_TOKEN:
 #ifdef SLANG_HAS_FLOAT
@@ -2406,12 +2737,11 @@ static void postfix_expression (_pSLang_Token_Type *ctok)
       case COMPLEX_TOKEN:
 #endif
       case STRING_DOLLAR_TOKEN:
-#ifdef HAVE_LONG_LONG
-      case LLONG_TOKEN:
-      case ULLONG_TOKEN:
-#endif
+      case MULTI_STRING_TOKEN:
 	append_token (ctok);
 	get_token (ctok);
+	if (ctok->type == OPAREN_TOKEN)
+	  _pSLparse_error(SL_SYNTAX_ERROR, "Literal constant is not callable", ctok, 1);
 	break;
 
       case OPAREN_TOKEN:
@@ -2520,14 +2850,14 @@ static void postfix_expression (_pSLang_Token_Type *ctok)
 
    while (_pSLang_Error == 0)
      {
-	end_pos = Token_List->len;
+	unsigned int end_pos = Token_List->len;
 	type = ctok->type;
 	switch (type)
 	  {
 	   case OBRACKET_TOKEN:	       /* X[args] ==> [args] X ARRAY */
 	     get_token (ctok);
 	     append_token_of_type (ARG_TOKEN);
-	     if (ctok->type != CBRACKET_TOKEN) 
+	     if (ctok->type != CBRACKET_TOKEN)
 	       array_index_expression (ctok);
 
 	     if (ctok->type != CBRACKET_TOKEN)
@@ -2550,14 +2880,18 @@ static void postfix_expression (_pSLang_Token_Type *ctok)
 		  last_token->type = _DEREF_FUNCALL_TOKEN;
 		  append_token_of_type (ARG_TOKEN);
 		  (void) get_token (ctok);
-		  function_args_expression (ctok, 0, 1);
-		  token_list_element_exchange (start_pos, end_pos);
+		  function_args_expression (ctok, 0, 1, 1, NULL);
+		  /* Now we have: ... @ __args ...
+		   * and we want: ... __args ... @
+		   */
+		  /* token_list_element_exchange (start_pos, end_pos); */
+		  token_list_element_exchange (end_pos-1, end_pos);
 		  break;
-	       } 
-	     
+	       }
+
 	     if (CPAREN_TOKEN != get_token (ctok))
 	       {
-		  function_args_expression (ctok, 1, 1);
+		  function_args_expression (ctok, 1, 1, 1, NULL);
 		  token_list_element_exchange (start_pos, end_pos);
 	       }
 	     else get_token (ctok);
@@ -2578,14 +2912,48 @@ static void postfix_expression (_pSLang_Token_Type *ctok)
 #ifdef DOT_METHOD_CALL_TOKEN
 	     if (ctok->type == OPAREN_TOKEN)
 	       {
+		  unsigned int qual_pos, meth_pos, x_pos, y_pos;
+		  /* This case is a bit tricky for expressions such as:
+		   *   foo(x;q1).bar(y;q2)
+		   *                ^
+		   * Here, '^' denotes the parse point.  The token list looks
+		   * roughly like:
+		   *    __arg x q1 __earg foo bar .
+		   */
 		  if (NULL == (last_token = get_last_token ()))
 		    return;
 		  last_token->type = DOT_METHOD_CALL_TOKEN;
-		  end_pos = Token_List->len;
+		  x_pos = start_pos;
+		  y_pos = Token_List->len;
+		  meth_pos = y_pos-1;
 		  append_token_of_type (ARG_TOKEN);
 		  get_token (ctok);
-		  function_args_expression (ctok, 0, 1);
-		  token_list_element_exchange (start_pos, end_pos);
+		  function_args_expression (ctok, 0, 1, 1, &qual_pos);
+		  if (_pSLang_Error)
+		    break;
+		  end_pos = Token_List->len;
+		  /* At this point, the token list looks like:
+		   *   __arg x q1 __earg foo methcall(bar) __arg y q2
+		   * x^                    m^            y^      q^
+		   * where ^ denotes the meth, x, y, and qual positions.
+		   * We want to rearrange this to be:
+		   *   __arg y __arg x q1 __earg foo q2 methcall(bar)
+		   * Do it in 3 stages:
+		   */
+		  token_list_element_exchange (x_pos, y_pos);
+		  /*   __arg y q2 __arg x q1 __earg foo methcall(bar)
+		   * y^      q^ x^                    m^
+		   */
+		  qual_pos = start_pos + (qual_pos-y_pos);
+		  meth_pos = meth_pos + (end_pos-y_pos);
+		  x_pos = start_pos + (end_pos-y_pos);
+		  token_list_element_exchange (qual_pos, x_pos);
+		  /*   __arg y __arg x q1 __earg foo methcall(bar) q2
+		   * y^      x^                    m^            q^
+		   */
+		  meth_pos = meth_pos - (x_pos - qual_pos);
+		  qual_pos = qual_pos + (end_pos-x_pos);
+		  token_list_element_exchange (meth_pos, qual_pos);
 	       }
 #endif
 	     break;
@@ -2622,10 +2990,14 @@ static void postfix_expression (_pSLang_Token_Type *ctok)
      }
 }
 
-static void function_args_expression (_pSLang_Token_Type *ctok, int handle_num_args, int handle_qualifiers)
+/* This function is used for more than function arguments */
+static void function_args_expression (_pSLang_Token_Type *ctok, int handle_num_args, int handle_qualifiers,
+				      int is_function,
+				      unsigned int *qual_posp)
 {
    unsigned char last_type, this_type;
-   
+   int has_qualifiers = 0;
+
    if (handle_num_args) append_token_of_type (ARG_TOKEN);
 
    last_type = (ctok->type == COMMA_TOKEN) ? COMMA_TOKEN : 0;
@@ -2645,8 +3017,15 @@ static void function_args_expression (_pSLang_Token_Type *ctok, int handle_num_a
 	   case CPAREN_TOKEN:
 	     if (last_type == COMMA_TOKEN)
 	       append_token_of_type (_NULL_TOKEN);
+
 	     if (handle_num_args) append_token_of_type (EARG_TOKEN);
+
+	     if ((qual_posp != NULL) && (has_qualifiers == 0))
+	       *qual_posp = Token_List->len;
+
 	     get_token (ctok);
+	     if (is_function && (ctok->type == OPAREN_TOKEN))
+	       _pSLparse_error (SL_SYNTAX_ERROR, "A '(' is not permitted here", ctok, 0);
 	     return;
 
 	   case SEMICOLON_TOKEN:
@@ -2655,6 +3034,11 @@ static void function_args_expression (_pSLang_Token_Type *ctok, int handle_num_a
 		  if (last_type == COMMA_TOKEN)
 		    append_token_of_type (_NULL_TOKEN);
 
+		  if (qual_posp != NULL)
+		    *qual_posp = Token_List->len;
+
+		  has_qualifiers = 1;
+
 		  if (SEMICOLON_TOKEN == get_token (ctok))
 		    {
 		       /* foo (args... ;; q) form */
@@ -2673,12 +3057,12 @@ static void function_args_expression (_pSLang_Token_Type *ctok, int handle_num_a
 		  break;
 	       }
 	     /* drop */
-		  
+
 	   default:
 	     simple_expression (ctok);
 	     if ((ctok->type != COMMA_TOKEN)
 		 && (ctok->type != CPAREN_TOKEN)
-		 && ((handle_qualifiers == 0) 
+		 && ((handle_qualifiers == 0)
 		     || (ctok->type != SEMICOLON_TOKEN)))
 	       {
 		  _pSLparse_error (SL_SYNTAX_ERROR, "Expecting ')'", ctok, 0);
@@ -2696,7 +3080,7 @@ static int check_for_lvalue (unsigned char eqs_type, _pSLang_Token_Type *ctok)
    if ((ctok == NULL)
        && (NULL == (ctok = get_last_token ())))
      type = ILLEGAL_TOKEN;
-   else 
+   else
      type = ctok->type;
 
    eqs_type -= ASSIGN_TOKEN;
@@ -2767,20 +3151,20 @@ static void array_index_expression (_pSLang_Token_Type *ctok)
 	     if (num_commas)
 	       _pSLparse_error (SL_SYNTAX_ERROR, "Misplaced ':'", ctok, 0);
 	     return;
-	     
+
 	   case TIMES_TOKEN:
 	     append_token_of_type (_INLINE_WILDCARD_ARRAY_TOKEN);
 	     get_token (ctok);
 	     break;
-	     
+
 	   case COMMA_TOKEN:
 	     _pSLparse_error (SL_SYNTAX_ERROR, "Misplaced ','", ctok, 0);
 	     return;
-	     
+
 	   default:
 	     simple_expression (ctok);
 	  }
-	
+
 	if (ctok->type != COMMA_TOKEN)
 	  return;
 	num_commas++;
@@ -2805,7 +3189,7 @@ static void inline_array_expression (_pSLang_Token_Type *ctok)
 
    if (ctok->type == COLON_TOKEN)	       /* [:...] */
      append_token_of_type (_NULL_TOKEN);
-   else if (ctok->type != CBRACKET_TOKEN) 
+   else if (ctok->type != CBRACKET_TOKEN)
      array_index_expression (ctok);
 
    if (ctok->type == COLON_TOKEN)
