@@ -1,6 +1,6 @@
 /* file intrinsics for S-Lang */
 /*
-Copyright (C) 2004-2011 John E. Davis
+Copyright (C) 2004-2010 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -818,7 +818,7 @@ static int build_dirlist (char *file, char *opt, char ***listp, unsigned int *nu
 
    num = 0;
    max_num = 50;
-   list = (char **)_SLcalloc (max_num, sizeof(char *));
+   list = (char **)SLmalloc (max_num * sizeof(char *));
    if (list == NULL)
      {
 	SLfree (pat);
