@@ -1,6 +1,6 @@
 /* SLang Screen management routines */
 /*
-Copyright (C) 2004-2011 John E. Davis
+Copyright (C) 2004-2010 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -1588,8 +1588,8 @@ static int init_smg (int mode)
    len = Screen_Cols + 3;
    for (i = 0; i < Screen_Rows; i++)
      {
-	if ((NULL == (old = (SLsmg_Char_Type *) _SLcalloc (len, sizeof(SLsmg_Char_Type))))
-	    || ((NULL == (neew = (SLsmg_Char_Type *) _SLcalloc (len, sizeof(SLsmg_Char_Type))))))
+	if ((NULL == (old = (SLsmg_Char_Type *) SLmalloc (sizeof(SLsmg_Char_Type) * len)))
+	    || ((NULL == (neew = (SLsmg_Char_Type *) SLmalloc (sizeof(SLsmg_Char_Type) * len)))))
 	  {
 	     SLfree ((char *) old);
 	     return -1;
