@@ -2,7 +2,7 @@
 /* Standard intrinsic functions for S-Lang.  Included here are string
    and array operations */
 /*
-Copyright (C) 2004-2009 John E. Davis
+Copyright (C) 2004-2010 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -19,7 +19,7 @@ General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-USA.  
+USA.
 */
 
 #include "slinclud.h"
@@ -71,31 +71,31 @@ static void do_dup(void) /*{{{*/
 
 /*}}}*/
 
-static int length_cmd (void)
+static void length_cmd (void)
 {
    SLang_Class_Type *cl;
    SLang_Object_Type obj;
    VOID_STAR p;
-   unsigned int length;
-   int len;
+   SLuindex_Type length;
+   SLindex_Type ilen;
 
    if (-1 == SLang_pop (&obj))
-     return -1;
+     return;
 
    cl = _pSLclass_get_class (obj.o_data_type);
    p = _pSLclass_get_ptr_to_value (cl, &obj);
 
-   len = 1;
+   ilen = 1;
    if (cl->cl_length != NULL)
      {
 	if (0 == (*cl->cl_length)(obj.o_data_type, p, &length))
-	  len = (int) length;
+	  ilen = (SLindex_Type) length;
 	else
-	  len = -1;
+	  ilen = -1;
      }
 
    SLang_free_object (&obj);
-   return len;
+   (void) SLang_push_array_index (ilen);
 }
 
 /* convert integer to a string of length 1 */
@@ -103,14 +103,14 @@ static void char_cmd (SLwchar_Type *x) /*{{{*/
 {
    SLuchar_Type buf[SLUTF8_MAX_MBLEN + 1];
    int is_byte;
-   
+
    is_byte = ((signed)*x < 0);
    if (is_byte)
      {
 	buf[0] = (SLuchar_Type) (-(signed)*x);
 	buf[1] = 0;
      }
-   else if ((_pSLinterp_UTF8_Mode == 0) 
+   else if ((_pSLinterp_UTF8_Mode == 0)
 	    || (*x < 0x80))
      {
         buf[0] = (SLuchar_Type) *x;
@@ -119,13 +119,13 @@ static void char_cmd (SLwchar_Type *x) /*{{{*/
    else
      {
         SLuchar_Type *p;
-        
+
         p = SLutf8_encode (*x, buf, SLUTF8_MAX_MBLEN);
         if (p == NULL) p = buf;
-        
+
         *p = 0;
      }
-   
+
    SLang_push_string ((char *)buf);
 }
 
@@ -189,11 +189,18 @@ static void intrin_getenv_cmd (char *s)
 }
 
 #ifdef HAVE_PUTENV
+/* This is a silly hack to deal with the ambiguity of whether or not to free
+ * a pointer passed to putenv.  Here it is attached to an array so that leak
+ * checkers can located it.
+ */
+# define MAX_PUTENV_ARRAY_SIZE 64
+static char *Putenv_Pointer_Array[MAX_PUTENV_ARRAY_SIZE];
+unsigned int Num_Putenv_Pointers = 0;
 static void intrin_putenv (void) /*{{{*/
 {
    char *s;
 
-   /* Some putenv implementations required malloced strings. */
+   /* Some putenv implementations require malloced strings. */
    if (SLpop_string(&s)) return;
 
    if (putenv (s))
@@ -201,8 +208,10 @@ static void intrin_putenv (void) /*{{{*/
 	SLang_set_error (SL_OS_Error);
 	SLfree (s);
      }
-
    /* Note that s is NOT freed */
+
+   if (Num_Putenv_Pointers < MAX_PUTENV_ARRAY_SIZE)
+     Putenv_Pointer_Array[Num_Putenv_Pointers++] = s;
 }
 
 /*}}}*/
@@ -325,13 +334,11 @@ static void usage (void)
    SLang_free_slstring (msg);
 }
 
-
 static void guess_type (char *s)
 {
    SLang_push_datatype (SLang_guess_type(s));
 }
 
-
 static int load_string_or_file (int (*f) (SLFUTURE_CONST char *, SLFUTURE_CONST char *))
 {
    char *file;
@@ -349,7 +356,7 @@ static int load_string_or_file (int (*f) (SLFUTURE_CONST char *, SLFUTURE_CONST
 	SLang_free_slstring (ns);
 	return -1;
      }
-   
+
    status = (*f) (file, ns);
    SLang_free_slstring (file);
    SLang_free_slstring (ns);
@@ -375,24 +382,55 @@ static int get_doc_string (char *file, char *topic)
    char *str;
    char ch;
 
+   topic_len = strlen (topic);
+   if (topic_len == 0)
+     return -1;
+
    if (NULL == (fp = fopen (file, "r")))
      return -1;
 
-   topic_len = strlen (topic);
-   ch = *topic;
-
    while (1)
      {
+	char *pos;
+
 	if (NULL == fgets (line, sizeof(line), fp))
 	  {
 	     fclose (fp);
 	     return -1;
 	  }
+	ch = *line;
+	if ((ch == ' ') || (ch == '\t') || (ch == '\n') || (ch == '-'))
+	  continue;
 
-	if ((ch == *line)
-	    && (0 == strncmp (line, topic, topic_len))
-	    && ((line[topic_len] == '\n') || (line [topic_len] == 0)
-		|| (line[topic_len] == ' ') || (line[topic_len] == '\t')))
+	pos = strstr (line, topic);
+	if (pos == NULL)
+	  continue;
+
+	ch = pos[topic_len];
+
+	/* Most common case */
+	if ((pos == line)
+	    && ((ch == '\n') || (ch == 0) || (ch == ' ') || (ch == '\t') || (ch == ',')))
+	  break;
+
+	pos = line;
+	while (NULL != (pos = strchr (pos, ',')))
+	  {
+	     /* Here *pos == ',' */
+	     if (NULL == (pos = strstr (pos+1, topic)))
+	       break;
+	     ch = pos[-1];
+	     if ((ch != ' ') && (ch != ',') && (ch != '\t'))
+	       {
+		  pos += topic_len;
+		  continue;
+	       }
+	     ch = pos[topic_len];
+	     if ((ch == '\n') || (ch == ',')
+		 || (ch == ' ') || (ch == '\t') || (ch == 0))
+	       break;
+	  }
+	if (pos != NULL)
 	  break;
      }
 
@@ -467,7 +505,7 @@ static void set_doc_files_intrin (void)
 
    if (-1 == SLang_pop_array_of_type (&at, SLANG_STRING_TYPE))
      return;
-   
+
    _pSLstring_list_delete (Doc_Files);
    Doc_Files = NULL;
 
@@ -491,14 +529,14 @@ static void get_doc_string_intrin (char *topic)
      {
 	if (-1 == SLang_pop_slstring (&file))
 	  return;
-	
+
 	if (-1 == get_doc_string (file, topic))
 	  (void) SLang_push_null ();
-	
+
 	SLang_free_slstring (file);
 	return;
      }
-   
+
    if ((Doc_Files == NULL)
        || (NULL == (files = Doc_Files->buf)))
      {
@@ -511,7 +549,7 @@ static void get_doc_string_intrin (char *topic)
 	file = files[i];
 	if (file == NULL)
 	  continue;
-	
+
 	if (0 == get_doc_string (file, topic))
 	  return;
      }
@@ -526,7 +564,7 @@ static int push_string_array_elements (SLang_Array_Type *at)
 
    if (at == NULL)
      return -1;
-   
+
    strs = (char **)at->data;
    num = at->num_elements;
    for (i = 0; i < num; i++)
@@ -541,7 +579,6 @@ static int push_string_array_elements (SLang_Array_Type *at)
    return 0;
 }
 
-	
 static void intrin_apropos (void)
 {
    int num_args;
@@ -556,7 +593,7 @@ static void intrin_apropos (void)
      return;
    if (-1 == SLang_pop_slstring (&pat))
      return;
-   
+
    namespace_name = NULL;
    at = NULL;
    if (num_args == 3)
@@ -691,16 +728,16 @@ static int do_obj_cmp_fun (int (*fun)(SLang_Object_Type *, SLang_Object_Type *))
 {
    int eqs;
    SLang_Object_Type a, b;
-   
+
    if (-1 == SLang_pop (&b))
      return -1;
-   
+
    if (-1 == SLang_pop (&a))
      {
 	SLang_free_object (&b);
 	return -1;
      }
-   
+
    eqs = (*fun) (&a, &b);
 
    SLang_free_object (&a);
@@ -718,7 +755,6 @@ static int eqs_intrinsic (void)
    return do_obj_cmp_fun (_pSLclass_obj_eqs);
 }
 
-
 static int is_callable_intrinsic (void)
 {
    SLang_Ref_Type *ref;
@@ -732,7 +768,7 @@ static int is_callable_intrinsic (void)
 
    if (-1 == SLang_pop_ref (&ref))
      return -1;
-   
+
    ret = _pSLang_ref_is_callable (ref);
    SLang_free_ref (ref);
 
@@ -741,14 +777,14 @@ static int is_callable_intrinsic (void)
 
 static int is_numeric (SLtype type)
 {
-   /* Version 2: Add attributes to the class tables to simplify this.  
+   /* Version 2: Add attributes to the class tables to simplify this.
     * Also clarify exactly what _pSLang_is_arith_type is supposed to return.
     */
    if (0 == _pSLang_is_arith_type ((SLtype) type))
      {
 	if (type == SLANG_COMPLEX_TYPE)
 	  return 3;
-	
+
 	return 0;
      }
    if ((type == SLANG_DOUBLE_TYPE) || (type == SLANG_FLOAT_TYPE))
@@ -760,7 +796,7 @@ static int is_numeric (SLtype type)
 static int is_numeric_intrinsic (void)
 {
    int type;
-   
+
    if (-1 == (type = SLang_peek_at_stack1 ()))
      return -1;
 
@@ -774,7 +810,7 @@ static int is_datatype_numeric_intrinsic (void)
 
    if (-1 == SLang_pop_datatype (&type))
      return -1;
-   
+
    return is_numeric (type);
 }
 
@@ -844,7 +880,7 @@ static void intrin_atof (void)
    strp = (char **) ats->data;
    strpmax = strp + ats->num_elements;
    ip = (double *) ati->data;
-	
+
    while (strp < strpmax)
      {
 	if (*strp == NULL)
@@ -880,7 +916,7 @@ static void intrin_integer (void)
    strp = (unsigned char **) ats->data;
    strpmax = strp + ats->num_elements;
    ip = (int *) ati->data;
-	
+
    while ((strp < strpmax) && (_pSLang_Error == 0))
      {
 	if (*strp == NULL)
@@ -915,7 +951,7 @@ static void atoi_intrin (void)
    strp = (char **) ats->data;
    strpmax = strp + ats->num_elements;
    ip = (int *) ati->data;
-	
+
    while (strp < strpmax)
      {
 	if (*strp == NULL)
@@ -949,7 +985,7 @@ static void atol_intrin (void)
    strp = (char **) ats->data;
    strpmax = strp + ats->num_elements;
    ip = (long *) ati->data;
-	
+
    while (strp < strpmax)
      {
 	if (*strp == NULL)
@@ -1040,7 +1076,7 @@ static void expand_dollar_string (char *s)
 static void get_qualifiers_intrin (void)
 {
    SLang_Struct_Type *q;
-   if (0 == _pSLang_get_qualifiers (&q))
+   if (0 == _pSLang_get_qualifiers_intrin (&q))
      (void) SLang_push_struct (q);
 }
 
@@ -1048,13 +1084,13 @@ static int qualifier_exists_intrin (char *name)
 {
    SLang_Struct_Type *q;
 
-   if (-1 == _pSLang_get_qualifiers (&q))
+   if (-1 == _pSLang_get_qualifiers_intrin (&q))
      return -1;
-   
+
    if ((q == NULL)
        || (NULL == _pSLstruct_get_field_value (q, name)))
      return 0;
-   
+
    return 1;
 }
 
@@ -1065,7 +1101,7 @@ static void qualifier_intrin (void)
    SLang_Struct_Type *q;
    SLang_Object_Type *objp;
 
-   if (-1 == _pSLang_get_qualifiers (&q))
+   if (-1 == _pSLang_get_qualifiers_intrin (&q))
      return;
 
    has_default = (SLang_Num_Function_Args == 2);
@@ -1082,7 +1118,7 @@ static void qualifier_intrin (void)
      objp = _pSLstruct_get_field_value (q, name);
    else
      objp = NULL;
-   
+
    SLang_free_slstring (name);
 
    if (objp != NULL)
@@ -1093,7 +1129,7 @@ static void qualifier_intrin (void)
      }
    else if (has_default == 0)
      (void) SLang_push_null ();
-   
+
    /* Note: objp and q should _not_ be freed since they were not allocated */
 }
 #endif
@@ -1174,7 +1210,7 @@ static SLang_Intrin_Fun_Type SLang_Basic_Table [] = /*{{{*/
    MAKE_INTRINSIC_S("implements", _pSLang_implements_intrinsic, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("use_namespace", _pSLang_use_namespace_intrinsic, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("current_namespace", _pSLang_cur_namespace_intrinsic, SLANG_STRING_TYPE),
-   MAKE_INTRINSIC_0("length", length_cmd, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("length", length_cmd, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("__set_argc_argv", set_argv_intrinsic, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("_$", expand_dollar_string, SLANG_VOID_TYPE),
 #if SLANG_HAS_QUALIFIERS
@@ -1306,7 +1342,7 @@ int SLang_init_slang (void) /*{{{*/
      return -1;
 #endif
 
-   if ((SLang_Doc_Dir != NULL) 
+   if ((SLang_Doc_Dir != NULL)
        && (*SLang_Doc_Dir != 0))
      {
 	char *docfile = SLpath_dircat (SLang_Doc_Dir, "slangfun.txt");
@@ -1344,21 +1380,21 @@ static int add_argc_argv (SLang_Array_Type *at)
 static void set_argv_intrinsic (void)
 {
    SLang_Array_Type *at;
-   
+
    if (-1 == SLang_pop_array_of_type (&at, SLANG_STRING_TYPE))
      return;
 
    if (-1 == add_argc_argv (at))
      SLang_free_array (at);
 }
-   
+
 int SLang_set_argc_argv (int argc, char **argv)
 {
    SLang_Array_Type *at = _pSLstrings_to_array (argv, argc);
 
    if (at == NULL)
      return -1;
-   
+
    if (-1 == add_argc_argv (at))
      {
 	SLang_free_array (at);
