@@ -1,6 +1,6 @@
 /* file stdio intrinsics for S-Lang */
 /*
-Copyright (C) 2004-2011 John E. Davis
+Copyright (C) 2004-2010 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -581,7 +581,7 @@ static void stdio_fgetslines_internal (FILE *fp, unsigned int n)
 	  max_num_lines++;
      }
 
-   list = (char **) _SLcalloc (max_num_lines, sizeof (char *));
+   list = (char **) SLmalloc (sizeof (char *) * max_num_lines);
    if (list == NULL)
      return;
 
