@@ -1,7 +1,7 @@
 /* -*- mode: C; mode: fold; -*- */
 /* string manipulation functions for S-Lang. */
 /*
-Copyright (C) 2004-2011 John E. Davis
+Copyright (C) 2004-2010 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -160,7 +160,7 @@ static void strcat_cmd (void) /*{{{*/
 
    if (nargs <= 10)
      ptrs = ptrs_buf;
-   else if (NULL == (ptrs = (char **)_SLcalloc (nargs, sizeof (char *))))
+   else if (NULL == (ptrs = (char **)SLmalloc (nargs * sizeof (char *))))
      return;
 
    memset ((char *) ptrs, 0, sizeof (char *) * nargs);
@@ -448,7 +448,7 @@ static void str_uncomment_string_cmd (char *str, char *b, char *e) /*{{{*/
 	return;
      }
 
-   if (NULL == (etable = (SLuchar_Type *) _SLcalloc (blen, (SLUTF8_MAX_MBLEN+1))))
+   if (NULL == (etable = (SLuchar_Type *) SLmalloc (blen * (SLUTF8_MAX_MBLEN+1))))
      return;
 
    b1 = (SLuchar_Type *) b;
@@ -2611,7 +2611,7 @@ static void create_delimited_string_cmd (int *nptr)
 
    n = 1 + (unsigned int) *nptr;       /* n includes delimiter */
 
-   if (NULL == (strings = (char **)_SLcalloc (n, sizeof (char *))))
+   if (NULL == (strings = (char **)SLmalloc (n * sizeof (char *))))
      {
 	SLdo_pop_n (n);
 	return;
@@ -2666,7 +2666,7 @@ static void strjoin_cmd (void)
    (void) SLang_push_malloced_string (str);   /* NULL Ok */
 }
 
-static unsigned int count_char_occurrences (char *str, SLwchar_Type *wchp)
+static unsigned int count_char_occurances (char *str, SLwchar_Type *wchp)
 {
    SLwchar_Type wch = *wchp;
    SLuchar_Type wch_utf8[SLUTF8_MAX_MBLEN+1];
@@ -2951,8 +2951,7 @@ static SLang_Intrin_Fun_Type Strops_Table [] = /*{{{*/
    MAKE_INTRINSIC_S("strtrans", strtrans_vintrin, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("str_delete_chars", str_delete_chars_vintrin, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("glob_to_regexp", glob_to_regexp, SLANG_VOID_TYPE),
-   MAKE_INTRINSIC_2("count_char_occurrences", count_char_occurrences, SLANG_UINT_TYPE, SLANG_STRING_TYPE, SLANG_WCHAR_TYPE),
-   MAKE_INTRINSIC_2("count_char_occurances", count_char_occurrences, SLANG_UINT_TYPE, SLANG_STRING_TYPE, SLANG_WCHAR_TYPE),
+   MAKE_INTRINSIC_2("count_char_occurances", count_char_occurances, SLANG_UINT_TYPE, SLANG_STRING_TYPE, SLANG_WCHAR_TYPE),
    MAKE_INTRINSIC_0("strskipbytes", skip_bytes_intrin, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("islower", islower_intrin, SLANG_CHAR_TYPE),
    MAKE_INTRINSIC_0("isupper", isupper_intrin, SLANG_CHAR_TYPE),
